// ********************************* Tilia.h **************************************

//
// Tilia Music Representation
//
// This was written by Lippold Haken and Dorothea Blostein.
//
// Copyright (c) Lippold Haken and Dorothea Blostein, 1993 - 2005.
//
// We encourage distribution of this C header file along with the
// Lime application and the Lime User's Manual.
//
// Incorporation of this C header file into other programs is encouraged.  The Lime
// authors do not retain any rights to such software.
//
// For an introduction to the Tilia Music Representation, see "The Tilia Music
// Representation: Extensibility, Abstraction, and Notation Contexts for the Lime
// Music Editor," Computer Music Journal, Vol 17, No 3, Fall 1993, pp.43-58.
//



#ifndef _TILIAH_
#define _TILIAH_


//
// This file contains the type definitions for the Tilia music representation.
// The Tilia music representation is used by the Lime music notation program.
// These type definitions contain the information you need to be able to read,
// modify or create Tilia files on disk.
//
// A Tilia file is written in "binary" (non-ascii) format as follows:
//		header
//		Sequence of Tilia nodes for first voice
//		Sequence of Tilia nodes for second voice
//		...
//		Sequence of Tilia nodes for last voice
// The header defines the number of voices in the piece, as well as other information.
// A complete description of the header contents, as well as Tilia node format,
// is given later in this file.
//
// As an overview, the format of the file header is
//		file-format number  (2 bytes)
//		H					(2 bytes)
//		other info			(H bytes)
// H is variable here; when H is small, the unspecified quantities are given default
// values by Lime when the piece is opened.  All pieces created by Lime are given
// full headers (H is large).  In the future, the header may be extendeded to contain
// more information; thus H will increase.
//
// The minimal header is six bytes long and consists of
//		file-format number  (2 bytes)		set this to CURRENT_FILE_FORMAT_NUMBER
//		2					(2 bytes)
//		no. voices			(2 bytes)
//

// ---------------------- CURRENT_FILE_FORMAT_NUMBER ----------------------------
//
// As a general principle, a version of Lime should always be able to read the
// next version of a Tilia file, albeit with the possibility of information loss.
// If a change to the format is made that will mean that the previous version of
// Lime cannot possibly read it, then the file-format-number should be increased
// by 2 (two) and, if practical, an interim version produced that can read it.
//
#define CURRENT_FILE_FORMAT_NUMBER	344

#define FIRST_56_BYTE_NODE_VERSION	343
#define GENERAL_RELATIVITY_VERSION	345	// Version number supporting general relativity
#define FIRST_52_BYTE_NODE_VERSION	347

#if CURRENT_FILE_FORMAT_NUMBER < GENERAL_RELATIVITY_VERSION
# define NEWEST_READABLE_INCREMENT	3	// Change back to 1 when version greater than general relativity
#elif CURRENT_FILE_FORMAT_NUMBER == GENERAL_RELATIVITY_VERSION
# define NEWEST_READABLE_INCREMENT	2	// Change back to 1 when version greater than general relativity
#else
# define NEWEST_READABLE_INCREMENT	1
#endif


//
// ---------------------- Generic type definitions ------------------------------
// Implementations of the C language vary in the number of bits that are allocated
// for types such as "int" and "long".  In order to avoid confusion, we declare
// macros for signed and unsingned integers of 8, 16 and 32 bits, as well as for
// floating point numbers of 32, 64 and 80 bits.
//
// Two sets of definitions are supplied here: for compiling Lime to run under
// Windows (using the proper Dynamic Link Library to supply toolbox-like routines),
// and for compiling Lime to run on a real Macintosh.
//

#if COMPILE_FOR_WINDOWS			// Definitions for imitation Mac under 32-bit Windows
	typedef signed		char	INT8;		// INT8	   -- signed	8-bit integer
	typedef unsigned	char	UNINT8;		// UNINT8  -- unsigned  8-bit integer
	typedef				short	INT16;		// INT16   -- signed   16-bit integer
	typedef unsigned	short	UNINT16;	// UNINT16 -- unsigned 16-bit integer
	typedef				int		INT32;		// INT32   -- signed   32-bit integer
	typedef unsigned	long	UNINT32;	// UNINT32 -- unsigned 32-bit integer
	typedef		long	long	INT64;		// INT64   -- signed   64-bit integer
	#define PCHAR		unsigned char		// PCHAR   -- pascal character
	typedef				void *	PSTRING;	// PSTRING -- pascal string

	typedef				float	REAL32;		// REAL32  -- 32-bit floating point number
	#define				VOID	void
	#define				LOCAL	static
	#define				HUGE
	#define				SYSCALLBACK FAR PASCAL
											// for Toolbox (DLL) callback functions

#else // COMPILE_FOR_INTELMAC || COMPILE_FOR_POWERMAC

	typedef				char	INT8;		// INT8	   -- signed	8-bit integer
	typedef unsigned	char	UNINT8;		// UNINT8  -- unsigned  8-bit integer
	typedef				short	INT16;		// INT16   -- signed   16-bit integer
	typedef unsigned	short	UNINT16;	// UNINT16 -- unsigned 16-bit integer
	typedef				long	INT32;		// INT32   -- signed   32-bit integer
	typedef unsigned	long	UNINT32;	// UNINT32 -- unsigned 32-bit integer
	typedef		long	long	INT64;		// INT64   -- signed   64-bit integer
	typedef unsigned	char	PCHAR;		// PCHAR   -- pascal character
	typedef unsigned	char *	PSTRING;	// PSTRING -- pascal string
	typedef				short	Coord;		// Coord   -- screen coordinate

	typedef				float	REAL32;		// REAL32  -- 32-bit floating point number
	#define				VOID	void
	#define				LOCAL	static
	#undef				HUGE				// LH122908 3.12 TGG [Big Pine Key]
	#define				HUGE
	#define				SYSCALLBACK pascal	// for Toolbox callback functions
	typedef				int BOOL;
	typedef				char * LPSTR;

#endif

// ---------------------------- sizes of variables ---------------------------
//
enum{
	MAX08	  = 127,						// biggest positive  8-bit number
	MAX16	  = 32767,						// biggest positive 16 bit number
	MAX32	  = 2147483647L,				// biggest positive 32 bit number
	BIG16	  = 32000,						// big 16 bit number; this can be negated
	BIG32	  = 2100000000L,				// big positive 32 bit number
	UNDEF08	  = 0x80,						// undefined  8-bit value
	UNDEF16	  = 0x8000,						// undefined 16-bit value
	UNDEF32	  = 0x80000000,					// undefined 32-bit value
	UNDEFXY	  = UNDEF32,					// undefined XYPOS  value
	MAXUN8	  = 255,						// biggest unsigned  8 bit number
	MAXUN16	  = 65535,						// biggest unsigned 16 bit number
	MAXUN32	  = (UNINT32)4294967295LL		// biggest unsigned 32 bit number	
};



// Structures for pairs of x and y values.
//
typedef struct{	INT8	x, y; }	xy_INT8s_type;
typedef struct{	INT16	x, y; }	xy_INT16s_type;
typedef struct{	INT32	x, y; }	xy_INT32s_type;
typedef struct{	REAL32	x, y; }	xy_REAL32s_type;


// ------------------------------- Global constants ----------------------------
//
#undef  NIL
#define NIL		0
#undef  TRUE
#define TRUE	1			// Guaranteed to be 1
#undef  FALSE
#define FALSE	0			// Guaranteed to be 0
#undef  INDEFINED
#define INDEFINED(t)	(~(((unsigned t) ((t) -1)) >> 1))	// Undefined value of an arbitrary integer type.

enum{
	MAXVC				= 64,		// maximum number of voices in a piece
	MAXPART				= 64,		// maximum number of parts in a piece
	MAXSTF				= 64,		// maximum number of staves on a page (divided among any number of systems).
	MAX_PART_STAVES		= 7,		// Maximum number of staves used by a single part
	
	STF_CHORD_LIMIT		= 200,		// maxmimum number of notes in a chord -- all layers on a staff
	MAX_CHORD			= 10,		// Maximum number of notes in a newly recorded chord
	MAX_ACC_COLUMNS		= 15,		// maximum number of levels of stacked accidentals
#if !COMPILE_FOR_MACOS
	MAXMEAS				= 10000,	// maximum number of measures for everyone else (absolute max is 32760).
#else
	MAXMEAS				= 1000,		// maximum number of measures for puny OS9 systems
#endif
	MAXNCONTEXTS		= (MAXVC+1),// maximum number of notation contexts: score + each voice

	NOSTAFF				= -1,		// the "staff number" for voices that are not printing now
	NOLIST				= 0,		// the list value if a voice is not listed (deactivated)
	NOPAGE				= -500		// "page number" for a non-existent page
};

// Sizes that depend on binary version.
enum{
	TiliaBytes			= 56,		// In version 347 and later, 4 fewer bytes will be written to disk
	OLD_TiliaBytes		= 24,		// before XYPOS_FORMAT
	BytesParamValue		= 14,		// param structure size
	OLD_BytesParamValue	= 10 		// before XYPOS_FORMAT
};
typedef INT32	XYPOS;

// Identification of Provenance of a Lime Tilia file.
//
enum{
	Tilia_from_UNKNOWN	= 0,
	Tilia_from_POWERMAC	= 1,
	Tilia_from_INTELMAC	= 2,
	Tilia_from_WINDOWS	= 3,
	
	Tilia_from_THIS
#if   COMPILE_FOR_POWERMAC		
	=  Tilia_from_POWERMAC
#elif COMPILE_FOR_INTELMAC
	=  Tilia_from_INTELMAC
#elif COMPILE_FOR_WINDOWS
	=  Tilia_from_WINDOWS
#else
	=  Tilia_from_UNKNOWN
#endif		
};



// ============================= Format of Tilia files ===========================
// Every Tilia file begins with header information that has the following format.
// You can create a Tilia file without including all of this information in the
// header.  See the documentation at the start of this file.
//
// In the future, the header may be extended to contain more information, so
// H may increase.  Do not attempt to add your own extensions to this header
// definition, since this will interfere with future header formats.
// If you need to include additional information in Tilia files, do this by
// defining a new kind of Tilia node (see VINFO_node  defintion for an example).
//
//
// The format of Tilia files on disk is:
//	Header  [starts at byte 0 in the data fork]
//		version number for format of Tilia nodes		(2 bytes)
//		H (number of header bytes - 4)					(2 bytes)
//		number of voices								(2 bytes)
//		page number; 0 is a special value; see
//		comments on MUST_SET_PERFORM definition below	(2 bytes)
//		N (number of notation contexts)					(2 bytes)
//		current editing notation context				(2 bytes)
//		T (total number of bytes in N options records)  (2 bytes)
//		OPTIONS RECORDs for N notation contexts			(T bytes; Record defn below)
//		R (number of bytes in performance record)		(2 bytes)
//		PERFORMANCE RECORD								(R bytes; Record defn below)
//		P (total number of bytes in N print records)	(2 bytes)
//		PRINT RECORDs for N notation contexts			(P bytes; Record defn in toolbox)
//		S (number of bytes in ncontext names record)	(2 bytes)
//		NOTATION CONTEXT NAMES RECORD					(S bytes, S/N bytes per name)
//	Music Binary  [starts at byte H + 4 (== T + R + P + S + 20)]
//		first Tilia node for first voice				(TiliaBytes bytes)
//		second Tilia node for first voice				(TiliaBytes bytes)
//		...												(a multiple of TiliaBytes bytes)
//		last Tilia node fir first voice					(TiliaBytes bytes)
//		first Tilia node for second voice (if any)		(TiliaBytes bytes)
//		...												(a multiple of TiliaBytes bytes)
//		last Tilia node for second voice (if any)		(TiliaBytes bytes)
//		first Tilia node for third voice (if any)		(TiliaBytes bytes)
//		...												(a multiple of TiliaBytes bytes)
//		...												(a multiple of TiliaBytes bytes)
//		last Tilia node of last voice					(TiliaBytes bytes)
//
//


//					   --- Tilia File Header: MUST_SET_PERFORM ---
// If the page number specified in the Tilia header equals "MUST_SET_PERFORM", this
// indicates that performance fields of the NOTE node (such as xlength, xttn,
// and xpitch) should be set to default values by Lime.  Lime computes the values
// from the notated duration and pitch using the same algorithms as the Recompute
// Playback menu option.
//
// If there are several lines of lyrics at the same location, they are moved apart
// so they do not overlap.  (This simplifies the OPAL to Tilia Translator program.)
//
// Lime will start editing page 1 after modifying the Tilia representation.
//
#define MUST_SET_PERFORM 0


//					  --- Tilia File Header: OPTIONS RECORD ---
// One options record is kept for each notation context in the current file format.
// This is unlike the perform record (defined below); only one perform record per piece.
//
typedef struct {
	XYPOS page_x;			// logical page size in x (in .1 inches)
	XYPOS page_y;			// logical page size in y (in .1 inches)
#define MIN_PAGE_SIZE 1.5	// minimum logical page size (in floating inches)
#define MAX_PAGE_SIZE 4000	// maximum logical page size (in floating inches);
							// this comes from 72 dots per inch and BIG32 as max x or y
	INT16 hide_init_bar_num;// TRUE avoids printing barline number at start of piece
	INT16 first_bar_num;	// first measure in piece has this number
	INT16 clef;				// TRUE -> make clefs automatically at start of system
	INT16 keysig;			// TRUE -> make keysigs automatically at start of system
	INT16 print_tuplets;	// TRUE -> print tuplet numbers
	REAL32 nv_spacing;		// "linearity" of chord spacing: 1 means all note
							// values get the same space, .1 means longer notes
							// get much much more space than shorter notes
	XYPOS odd_left_points;	// points to left of staff lines, odd pages (tenth point units)
	XYPOS first_left_points;// points to left of staff lines in first system
	XYPOS odd_right_points; // points to right of staff lines, odd pages (tenth point units)
	XYPOS last_right_points;// points to right of staff lines in last system
	INT16 print_page_nums;	// TRUE if page numbers should be printed
	INT16 first_page_num;	// page number for first pagee in the piece
	INT16 XXXsystem_bracket;// obsolete -- replaced by xbracket field in ENDSYST node
	INT16 XXXmac_speaker;	// obsolete
	INT16 XXXmidi_out;		// obsolete
	INT16 XXXmidi_echo;		// onsolete -- now in prefs record
	XYPOS above_points;		// points above top staff line (tenth point units)
	XYPOS first_above_points;// points above top staff line on first page
	XYPOS below_points;		// points below bottom staff line (tenth point units)
	XYPOS first_below_points;// points below bottom staff line on first page
	INT16 linear_space;		// TRUE to use linear spacing; FALSE for normal spacing
	INT16 print_piece_name; // TRUE to print the piece name at bottom of page
	INT16 XXXprint_hidden;	// obsolete -- now in prefs record
	INT16 three_adjacent;	// TRUE if playing 3 adjacent notes on keyboard should
							// create a rest
	INT16 XXXzoomSF;		// obsolete -- scale factor for editing zoom; see REAL32 vers below	  9.11
	INT8  resizable;		// Music window is resizable.							// JA20111128 4.1.11b
	INT8  XXXscreen_hidden; // obsolete -- now in prefs record;
	INT16 XXXalt_color;		// obsolote -- use alternate xnote_color field values
	INT16 print_color;		// TRUE to print color noteheads on paper
	INT16 sys_sep;			// TRUE if system separation marks are to be drawn
	INT16 noNoteChanges;	// TRUE if changing notes is disabled; annotations changes ok
	INT16 minlw;			// minimum default line width
	XYPOS even_left_points; // points to left of staff lines, even pages (tenth point units)
	XYPOS even_right_points;// points to right of staff lines, even pages (tenth point units)
	REAL32 zoomSF;			// scale factor for editing zoom; STD_SF means no zoom	 9.11REAL32
	INT16 bPublished;		// allow changes to music (or allow only markup)?
	// If you add elements here, remember to addto sizes string below.
} options_struct;
//
// This string specifies the number of bytes for each member of options_struct.
// This information is used to reorder bytes when COMPILE_FOR_WINDOWS; Windows has
// the low-order data byte first in memory, where Macs have the high order data
// byte first.
//
#define options_struct_sizes "xx222224xxxx222222xxxx222221122222xx42"



//					 --- Tilia File Header: PERFORMANCE RECORD ---
// Only one perform record is kept per Tilia file; this is unlike the options record,
// since each notation context has its own options record.  Most entries in the perform
// record have to with playback parameters, but not all.
//
typedef struct {
	INT16 stacclength;	// percent of written note value played for staccato; 0-200%
	INT16 tenutolength; // percent of written note value played for tenuto; 0-200%
	INT16 normlength;	// percent of written note value played for normal note; 0-200%
	INT16 grace_onbeat; // TRUE if grace notes sound on the beat;
						// FALSE if they sound before the beat
	INT16 grace_length; // Grace note length, as a percent of 16th note length.
						// Values range from 0 to 400 (400 equals a quarter note)
		// The following fields control the amount of change effected by the
		// commands in the Performance menu.
	INT16 delta_volume; // volume change (increment/decrement for volumes
						// between 0..200)
	INT16 delta_pitch;	// pitch change (half-cents)
	INT16 delta_length; // length change as a percent of current played length
	INT16 delta_start;	// start-time change, as a percent of a 16th note

	INT8  kludge_flags;	// Header flags - temporarily here until next file version.
	
	INT8  delta_obsol;	// obsolete delta_nuance (was 16 bits)	
	
	INT16 obsolete[12]; // obsolete tuning table: half-cents offset for each mod12
						// pitch in the scale; now tuning_table[][] is used instead
	INT16 noeffect_lth; // TRUE if staccato, tenuto, slurs have no effect on
						// played note attributes; in this case stacclength, tenutolength
						// and normlength are irrelevant.
	INT16 dynamic_level[10];
						// default level used for dynamic markings pppp ... ffff
	INT16 noeffect_dyn; // TRUE if dynamic markings have no effect on volume levels;
						// in this case, dynamic_level[] is irrelevant.
	INT16 slurlength;	// percent of written note value played for slurs; 0-200%
	REAL32 tuning_table[7][6];
						// cents offset for each notated pitch in the octave;
						// first index is mod7 note in octave (0=c, 1=d, ... 6=b)
						// second index is xacc (1=--, 2=-, 3=n, 4=+, 5=++)

#define USER_COLORS 16	// number of user-defined colors in preferences file
	RGBColor ucolor[USER_COLORS];	// copy of user_color[] array (see definition below)
} perform_struct;
//
// This string specifies the number of bytes for each member of perform_struct.
// This information is used to reorder bytes when COMPILE_FOR_WINDOWS; Windows has
// the low-order data byte first in memory, where Macs have the high order data
// byte first.
//
#define perform_struct_sizes "222222222112222222222222222222222222\
444444444444444444444444444444444444444444222222222222222222222222222222222222222222222222"




/*					  --- Tilia File Header: PRINT RECORD ---
   One LimePrintRecored is kept for each notation context in the current file format.
   This is unlike the perform record (defined above); there is only one perform
   record per piece.
*/

typedef struct
{
	TPrint tprint;			// obsolete pre-carbon print record; here for easy backwards compatability; 120 bytes long
	INT32 percent_scale;	// percent reduction/enlargement
	INT32 landscape;		// true for landscape paper orientation
	INT32 paper_top;		// physical paper rectangle, in points!
	INT32 paper_left;
	INT32 paper_bottom;
	INT32 paper_right;
} LimePrintRecord;
#define print_struct_sizes "22222222222222211222222222222111111222114421144444444411444444"	// A gross of bytes
//	shorts in struct:		123456789012345  678901234567      890    1  *          #
//	bytes up to *:  31*2 + 2*4 + 12*1 = 82.  36 empty bytes (to 118), then 2 bytes (to 120) at # to fill out TPrint.



// ------------------------- Tilia Preferences Structure -----------------------------
//
// This structure contains the infromation from the Lime's Preferences file on the Mac,
// or the LIME.INI file under windows.
//
// On the Mac, the Prefences file stores this structure directly.  A default copy of
// the Preferences is kept in Lime's resource fork, for initial creation of the Preferences
// file.
//
// Under Windows, the LIME.INI file stores this information in a standard text format.
//

typedef struct {					// structure for user's command and control key settings
	INT16	menu;					// menu for command/control key option
	INT16	item;					// menu item for command/control key (0 means not used)
} uc;

enum { play_QuickTime = 0, play_MIDIManager = 1, play_CoreMIDIosx_OMSos9 = 2 };
typedef struct {
	INT32 versName;					// Lime version name for this preferences file (4-character string)
	INT16 play_method;				// play_MIDIManager, play_CoreMIDIosx_OMSos9, or play_QuickTime
	INT8  unusedx;
	INT8  midi_echo;				// TRUE to output sound when midi is input
	INT16 xxx2;						// placeholder (do not use!)

#define UCOMMAND_KEYS 26			// one command key entry for each letter a..z
	uc ucommand[UCOMMAND_KEYS];		// user's command keys; map letter to menu/item number

	INT16 default_music_font;		// default font for music, as defined below

	INT16 save_file_reminder;		// TRUE to be asked to save file on page change

	INT16 xxxkpad_box[32];			// obsolete; see pianoeq_loc below

	RGBColor user_color[USER_COLORS];// user-defined colors for color dialogs
#define BLACK_COLOR_INDEX 0			// user_color[0] is always black
#define MODIFIED_COLOR_INDEX 15		// editor indicates xmodvol/xmodp

#define UCONTROL_KEYS 26			// one control key entry for each letter a..z
	uc ucontrol[UCONTROL_KEYS];		// user's command keys; map letter to menu/item number

	UNINT8	disp_tempo : 1;			// TRUE to display hidden tempo markings
	UNINT8	disp_dynamic : 1;		// TRUE to display hidden dynamic markings
	UNINT8	disp_midi : 1;			// TRUE to display hidden midi text annotations
	UNINT8	disp_param : 1;			// TRUE to display hidden parameter text annotations
	UNINT8	disp_other : 1;			// TRUE to display other hidden text annotations
	UNINT8	disp_pedal : 1;			// TRUE to display hidden pedal markings
	UNINT8	disp_fing : 1;			// TRUE to display hidden fingerings
	UNINT8	notusedyet : 1;

	UNINT8	print_tempo : 1;		// TRUE to print hidden tempo markings
	UNINT8	print_dynamic : 1;		// TRUE to print hidden dynamic markings
	UNINT8	print_midi : 1;			// TRUE to print hidden midi text annotations
	UNINT8	print_param : 1;		// TRUE to print hidden parameter text annotations
	UNINT8	print_other : 1;		// TRUE to print other hidden text annotations
	UNINT8	print_pedal : 1;		// TRUE to print hidden pedal markings
	UNINT8	print_fing : 1;			// TRUE to print hidden fingerings
	UNINT8	unusednow : 1;

	Point PWinLoc;					// origin for the PianoWindow; see below for size

	UNINT8	color_modvol : 1;		// TRUE to color notes with xmodvol set
	UNINT8	color_modp : 1;			// TRUE to color notes with xmodp set
	UNINT8	color_pname : 1;		// TRUE to color notes according to note name
	UNINT8	unusedxxx : 5;

	INT8	metrochan;				// MIDI channel number for metronome
	UNINT16 metropitch;				// xpitch value for metronome
	
	INT8	use_postscript;			// (*hook for future*) on PowerMacs only, default use of postscript printing 
	UNINT8	minpix;					// on PowerMacs only, the minimum number of pixels in PostScript linewidth

	INT8	default_zoom : 6;		// Default zoom for new pieces in .5x steps. >20 = fit width; 0 or -32 = normal; -ve = resizable.
	INT8	unusedxxxx : 2;
	
	INT8	save_file_backup : 2;	// (*hook for future*) TRUE to create backup when saving; TRUISH if only when saving, not save-as 
	UNINT8	music_appearance : 2;	// (*hook for future*) How music window is displayed (normal, black on white, white on black, etc)
	INT8	backslash_always : 2;	// (*hook for future*) 1 to always use backslashes [-1 never, reserved]; -2 don't always use backslashes; 0 use default. 
	
	UNINT8  rulers : 2;				// TRUE to show rulers

	INT32	MidiInput;				// input selected for OMS or CoreMIDI
	INT32	MidiOutput;				// ouput selected for OMS or CoreMIDI
	
	INT8	inused1;
	INT8	swapTabSpace;			// swap function of tab and space for lyrics
	
	INT8	inused2;
	UNINT8	chordDragType;			// if value is CMOVE, then generate CMOVE nodes with
									// editor's Chord Drag option; if ZONE, generate
									// ZONE nodes; if RMOVE, generate RMOVE nodes.
	INT16 staffDragType;			// operation of staff drag, value defined below.
#define DRAG_ONE_STAFF 333			// value for prefs->staffDragType
#define DRAG_SYSTEM 444				// value for prefs->staffDragType
#define DRAG_REMAINDER 555			// value for prefs->staffDragType

	INT16 freeRunsLeft;				// countdown of free runs of Lime
#define freeRuns 32040				// they get this many free runs of Lime
#define freeRunsWarning 40			// warn when we get down to this few free runs left

#define PIANOEQ_MAX 512				// size of pianoeq_loc array
#define PEQshift 0x100				// top bit of index indicates shift on PC
	Point pianoeq_loc[PIANOEQ_MAX]; // alphanumeric keyboard equivalent to mouse click in piano window;
									// indexed by alphanumeric code on mac or
									// virtual key code plus shift bit in Windows,
									// value is in pianowindow coords;
									// 0,0 means unused; set by Piano Window Keys dialog.
	Point PWinSize;					// PianoWindow size; see above for location.
	INT32 midiDivisions;			// divisions per quarter in Export MIDI
	INT32 simpMus;					// TRUE for simplified music notation display
	INT32 xxxscanNum;				// obsolete: Liszt scan number
	INT32 xxxscanFiles;				// obsolete: total number of files to scan
} prefs_struct;
//									// JA20111122 4.1.12 (also changed last 3 items from int to INT32 to guarantee 4 bytes long)
// If anything changes in above, modify MACLIME.CPP [until we change to character based preference files on Mac, like on Windows]

enum {	USE_MARL_TUFA = 0,			// default_music_font value: use MARL and TUFA font
		USE_SONATA = 1				// default_music_font value: use the SONATA font
};


// ----------------------------- stem directions -----------------------------
// These constants are used for stem directions, slur directions etc.
//
#define NONE		0		// value to indicate no slur
#define DOWN		1		// value to indicate stem/slur down
#define UP			2		// value to indicate stem/slur up
#define UPORDOWN	3		// value before the stem/slur direction has been
							// decided on.
#define NOTDIR(d) (((d) == 3) ? 3 : 3 - (d))



// ------------------------- notehead parentheses ----------------------------
// These constants are used for parentheses around noteheads.
// NONE is defined above
//
#define RIGHT		1		// parenthesis to right of notehead
#define LEFT		2		// parenthesis to left of notehead
#define BOTH		3		// parentheses on both sides of notehead

// ------------------------- page related ----------------------------
//
#define ODD			1
#define EVEN		2
#define TOP			1
#define BOTTOM		2

// ------------------------------- note values -------------------------------
// Note values have 16 integer bits, 16 fraction.
//
#define HALF	  0x08000L	// value of a half note
#define WHOLE	  0x10000L	// value of a whole note
#define BREVE	  0x20000L	// value of a breve note (a double-whole note)
#define DBREVE	  0x30000L	// value of a dotted breve note
#define DDBREVE	  0x38000L	// value of a double dotted breve
#define DDDBREVE  0x3C000L	// value of a triple dotted breve


// ------------------------------- coordinates --------------------------------
// Constants to convert from internal x,y representation to screen coordinates.
// Note that, currently, screen resolution is only perfect at zoom factors that
// are exact multiples of the normal (2x, 3x, ..., 10x).  Furthermore, there are
// currently annotation alignment problems for all zooms other than those that
// are multiples of 2 (i.e. 1x(10.0), 2x(5.0), 4x(2.5) and 8x(1.25)).									// JA20111128 4.1.11b
//
// On a Macintosh there is an absolute (QuickDraw) limit to the zoom possible on the
// screen of 4095 pixels in either direction (X or Y), beyond that it will crash.
// The maximum zoom is, therefore, score dependent.
//
#define STD_SF		  10.0					// Tilia's x,y locations are kept to tenth point unit accuracy (at standard no-zoom display size).
#define STD_SF_int	  10

#define MAXIMUM_SF	  (REAL32)1.0			// Absolute Maximum zoom
#define DOUBLESIZE_SF (REAL32)(STD_SF/2.0)	// curSF value for 'double' zoom setting
#define ENLARGE_SF	  (REAL32)(STD_SF/1.5)	// curSF value for 'enlarge" zoom setting (n.b. not perfect)// JA20111128 4.1.11b
#define FULLSIZE_SF	  (REAL32)1.0			// curSF value for full size
#define HUGESIZE_SF	  (REAL32)(STD_SF/4.0)	// curSF value for 'Huge size" zoom setting
#define MINISIZE_SF	  (REAL32)(STD_SF/0.375)// curSF value for 'mini' zoom setting (minimum usable)
#define NORMALSIZE_SF (REAL32)STD_SF		// curSF value for 'normal' zoom setting
#define REDUCESIZE_SF (REAL32)(STD_SF/0.75)	// curSF value for 'reduce' zoom setting
#define SUPERSIZE_SF  (REAL32)(STD_SF/8.0)	// curSF value for 8x

#define MINIMUM_SF	  (REAL32)1000.0		// 1%; only usable as a place holder						// JA20111128 4.1.11b

#define GENERIC_FITWIDTH_SF 0.0				// Generic value for flagging resizable fit-width			// JA20111128 4.1.11b

// Coordinate indicating "continue at current location".
// Lime does not internally generate these, except in Graphics Elements
// because it makes Allow Any Option-click processing impossibly messy;
// [the value -1111 used to be used by the old OPAL->Lime translator].
//
enum{ ADJACENT = -32768 };



// --------------------------- Tilia font numbers --------------------------------
//
// Certain fonts numbers are remapped in the Tilia representation to ensure
// consistency among different Macs.  (Fonts will have different font numbers on
// different Macs.)
//
// Hidden text nodes provide the font names for other font numbers used by TPARAM nodes.
//
enum{
	TILIA_0			=  0,
	TILIA_MARL		= -1,	// special Tilia font number for Marl font
	TILIA_TUFA		= -2,	// special Tilia font number for Tufa font
	TILIA_SONATA	= -3,	// special Tilia font number for Sonata font
	TILIA_GENEVA	= -4,	// special Tilia font number for Geneva font
	TILIA_TIMES		= -5,	// special Tilia font number for Times font
	TILIA_SYMBOL	= -6,	// special Tilia font number for Symbol font
	TILIA_SIMPLE	= -7	// special Tilia font number for Simplified notation font (reserved)
};

// ------------------------ character mapping typedefs ----------------------------
//

typedef struct {		// struct for the information needed for character mapping
	INT16 font;			// font number; certain font numbers remapped from standard
						// Mac numbers: TILIA_MARL, TILIA_TUFA, TILIA_SONATA,
						// TILIA_GENEVA, TILIA_TIMES, TILIA_SYMBOL
	UNINT8 charset;		// Windows charset number (for foreign languages)
	UNINT8 size;		// point size
	Style face;			// bold, italic, underline, etc
	char ch;			// the character
	XYPOS xoff, yoff;	// x and y offset for centering the character
} CHAR_SPEC;


typedef struct {		// struct for the information needed for short character strings;
						// this is not used by the Lime program yet
	INT16 font;			// font id
#define STR_MAX 8		// maximum string length in STR_SPEC
	char str[STR_MAX];	// the character string, teminated with '\0'
} STR_SPEC;


typedef union {			// union for storing PARAM information: information
						// about either a character, an integer, or a float.
	CHAR_SPEC cvalue;	// if it is a character parameter, this provides the
						// character specification
	STR_SPEC svalue;	// if it is a string parameter, this provides the
						// string specification
	REAL32 fvalue;		// if it is a floating parameter, this provides the float
	XYPOS ivalue;		// if it is an integer parameter, this provides the integer;
						// if it is a coordinate parameter, this provides the coord
						// (the user always sees coordinates divided by STD_SF)
} PARAM_VALUE;
//
// These strings specify the number of bytes for each member of PARAM_VALUE.
// This information is used to reorder bytes when COMPILE_FOR_WINDOWS; Windows has
// the low-order data byte first in memory, where Macs have the high order data
// byte first.
//
#define CHAR_SPEC_sizes		"21111xx"
#define STR_SPEC_sizes		"211111111"
#define REAL32_sizes		"4"
#define XYPOS_sizes			"x"
#define INT16_sizes			"2"		// not used by PARAM_VALUE at this time


// -------------------------------- notation contexts ---------------------------
// A notation-context is used to define the pieces parts: score, flute part, violin
// part etc.  For each notation context, there is a set of voices that appear somewhere
// in the notation-context.  For example, a violin part uses the violin voice, and may
// also use other voices (such as the flute voice, if there is a flute cue somewhere).
// By definition, a voice appears in notation context k if it there is a PRINT node with
// Xcontext == k before the first NOTE node in that voice.
//
// Each node in the binary has an xcontext field, indicating which notation contexts
// should pay attention to that node.
//
#define ALL_NCONTEXTS 0			// indicates node is for all notation contexts
#define SCORE_NCONTEXT 1		// notation context 1 is the score and always
								// has all voices active
#define FIRST_PART_NCONTEXT 2	// notation context for the first part
#define NOT_IN_SCORE_NCONTEXT -1
								// indicates node is for all notation conetxts except
								// SCORE_NCONTEXT



// --------------------------- varieties of annotations ------------------------------
// When a user modifies a Lime graphic using the Allow Any Option-click menu option,
// Lime generates LINE or TPARAM/TEXT nodes which contain the modified graphic.  Each
// time Lime reformats the page, it must identify the graphic and substitute the modified
// version.  This identification is done using the variety (defined in the enum below), and
// the oridinal of this variety associated with this note.
//
// A variety 0 (FALSE) indicates "normal" annotation.  Values 1 through 3 are not used
// (reserved for indicating hidden annotations internally in Lime subroutine arguments).
enum {
	vtie = 4, vslur, vtupl, vfretn, vbarn, vpagen, vstaffln, vpianotabstaff,			//	4-11
	vsysline, vsysbtop, vsysbleft, vsysbbetw, vsysbbot, vmainbeam, vsubbeam,			// 12-18
	vstemdown, vstemup, vbeamstemdown, vbeamstemup, vpianoroll, vledgerln,				// 19-24
	vpianotabledger, vbar1, vbar2, vbar3, vbarbetw1, vbarbetw2, vbarbetw3,				// 25-31
	vtopwingright, vtopwingleft, vbotwingright, vbotwingleft, vferm, vaccent,			// 32-37
	vstacmar, vbracetop, vbraceabove, vbracemid, vbracebelow, vbracebot, vmultirestn,	// 38-44
	vnhead, vrest, vdurdot, vacc, vupflag, vdnflag, vtrem, vclef, vtime, vcut,			// 45-54
	vcommon, vrepeatleft, vrepeatright, vkey, vhairpin1, vhairpin2, vtupbracknum,		// 55-61
	vtupbrackleftvert, vtupbrackrightvert, vtupbracklefthoriz, vtupbrackrighthoriz,		// 62-65
	vnotebrackleftvert, vnotebrackrightvert, vnotebrackhoriz, vfretl, vfing,			// 66-70
	vbend, vparen, vgliss, vbendrange, vslash, vsyssep, vrectvert1, vrectvert2,			// 71-78
	vrecthoriz1, vrecthoriz2,															// 79-
	
	variety_TOTAL
};



// --------------------- Typedefs for various kinds of Tilia nodes ---------------------
// Type definitions for the various kinds of music binary nodes.
// All Tilia nodes are exactly TiliaBytes bytes long.  This constant size is used for memory
// allocation purposes; some bytes are unused for certain node kinds.
// The first two bytes are common to all node kinds:
//	  - The first byte is Xkind, the kind of the node. (See the list of node kinds below.)
//	  - The second byte is Xcontext, indicating which notation contexts this node
//		is relevant to. Nodes with Xcontext==ALL_NCONTEXTS apply in all notation contexts.
//		Nodes with Xcontext==NOT_IN_SCORE_NCONTEXT apply in all notation contexts except
//		the score.  Other nodes apply only in the notation context whose number matches
//		the Xcontext value; the Xcontext value SCORE_NCONTEXT is reserved for the score.
//
// For internal Lime processing, the Tilia nodes are represented as a linked list;
// a 4-byte Xnext and a 4-byte Xback pointer are added to each node.  Internally to
// Lime this brings the size of Tilia nodes up to 32 bytes.  In Lime, a LINKS macro
// definition precedes "#include Tilia.h"; the LINKS macro creates the Xnext and Xback
// pointers. By default the following code leaves the LINKS macro defined to be null.
//

#define DATA_BYTES_IN_A_TILIA_NODE (TiliaBytes - 2)
#ifndef LINKS
	#define LINKS
	typedef void *DefTiliaPtr;
#else
	typedef union TEMPTilia HUGE *DefTiliaPtr;	// forward version of TiliaPtr type
#endif

//								--- node kinds ---
// The standard Tilia definition uses the node kinds listed below.  Many of
// these are node kinds which are stored on disk.  Lime uses additional node kinds
// internally that are never stored on disk; those are labeled "Temp Node" here.
// Nodes labeled GE here are graphics element nodes; these do not have an xcontext field.
//

#define NEST	76		// nested tuplet
#define CDDATA3 77		// Temp Node: Extension to chord data node
#define BARSEP	78		// companion to BAR node: indicates no bar drawn from staff to staff+1
#define XMLT	79		// Temp Node: For MusicXML import: timing from MusicXML file
#define COPY	80		// Temp Node: For copy-paste operation: separates voices in linked list
#define STMOD	81		// Tie and Slur modification info
#define xxxNIFFX 82		// obsolete: was X-location from NIFF import from scanning software
#define xxxFNAME 83		// obsolete: font name associated with a font number; now with hidden text
#define VOICES	84		// Temp Node: For Voices... dialog: info on voices which print together
#define NOTEPTR 85		// Temp Node: pointer to assoicated note, used in GE list
#define PLAY	86		// Temp Node: For time-ordered linked lists of Hear option PLAY nodes
#define ROD		87		// Temp Node: For linked list of rods in relocation pass
#define SPRING	88		// Temp Node: For linked list of springs in relocation pass
#define UNDO	89		// Temp Node: For linked list of Undo and Redo changes
#define MARK	90		// Temp Node: Separates Undo and Redo changes
#define COLOR	91		// color for notehead
#define RNOTE	92		// Temp Node: Used in Record and Import MIDI menu options
#define GE_CHAR	  93	// Graphics Element: character		(has no xcontext field)
#define GE_STRING 94	// Graphics Element: text string	(has no xcontext field)
#define GE_LINE	  95	// Graphics Element: line			(has no xcontext field)
#define GE_CURVE  96	// Graphics Element: curve			(has no xcontext field)
#define RMOVE	97		// move rest up or down
#define SPACC	98		// extra or special accidentals for note
#define AMOVE	99		// move accidental right or left
#define VINFO	100		// voice information
#define NOTE	101		// note or rest
#define CLEF	102		// clef
#define KEYSIG	103		// key signature
#define TIMESIG 104		// time signature
#define BAR		105		// bar line
#define FREE	106		// unused node (on free list)
#define TPARAM	107		// parameters for text: location, style, font
#define TEXT	108		// text
#define TTEXT	108		// text
#define LINE	109		// lines or curves
#define PARAM	110		// parameter set: character mapping and formatting constants
#define TEMPO	111		// Obsolete: Tempos are now specified by annotations
#define NTDATA	112		// Temp Node: Note data node (temporary extension to NOTE node)
#define CDDATA	113		// Temp Node: Chord data node (temporarily after last note in chord)
#define CDDATA2 114		// Temp Node: Extension to chord data node
#define ENDSYST 115		// end of system / end of page / end of piece
#define PRINT	116		// print formatting node
#define SDRAG	117		// drag the staff up or down
#define NA		118		// NA stands for "not applicable".	This is a fake node kind
						// that is used internally to Lime to indicate a mismatch
						// of notation context (based on the Xcontext value).
#define BEAMING 119		// beaming values
#define PDATA	120		// Temp Node: Position-data node
#define SPACE	121		// blank space -- supported only for backward compatability
#define MIDI	122		// Temp Node: Used in Record menu option; raw MIDI data
#define CMOVE	123		// move the chord right or left; affects no other chords
#define ZONE	124		// this chord is in a new relocation zone; specifies new
						// location for the chord, and relocates chords before and
						// after appropriately
#define NTVIEW	125		// Temp Node: Note data for another view
#define PVIEW	126		// Temp Node: position-data for another view


#define FIRST_Tilia_KIND NEST	// see also Tilia_kind_names[], Tilia_member_sizes[],
								// and the Tilia union definition in LnkTilia.h.
#define LAST_Tilia_KIND PVIEW	// do not use over 127 or INT8 values overflow!!

#define FIRST_GE GE_CHAR		// first GE type (GE types have no xcontext)
#define LAST_GE GE_CURVE		// last GE type	 (GE types have no xcontext)

//								--- node definitions ---
// There is one "struct" definition for each Tilia node kind.  Each struct definition
// begins with an Xkind and Xcontext field, as described above.
//
// In Lime it is convenient to include all of these "struct" definitions as
// part of a large "union" definition.
//
// Each struct also has a "sizes" string which specifies the number of bytes for
// each member of the struct, and the bitfield widths for any bitfield definitions
// in the struct. This information is used to reorder bytes when COMPILE_FOR_WINDOWS.
// Windows has the low-order data byte first in memory, where Macs have the high order
// data byte first.  Windows also assigns bitfields right to left in a byte, where Macs
// assign them left to right.
//
// ******************************************************************************
// *																			*
// * It is absolutely crucial that every structure subject to such 'endian'		*
// * conversion is properly aligned with its "sizes" string. Also, because many *
// * compilers have a habit of aligning multi-byte fields to 2 or 4 byte,		*
// * boundaries unused bytes should be inserted to pad to the required byte		*
// * alignment to accommodate them.												*
// *																			*
// ******************************************************************************
//

//								=== VINFO ===
// The VINFO node contains information about the voice, such as the part name.
// The VINFO node should follow the ENDSYST node that starts the voice (the
// ENDSYST node that has xstartpiece==TRUE).
//
#define MAXPNAME 16				// max number of characters in a part name (including '\0')
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is VINFO
	INT8	Xcontext;			// defines the notation contexts this node applies to

	char	Xpname[MAXPNAME];	// Part name, including the terminating '\0'
	INT8	Xmidichan;			// MIDI channel number for the voice
	UNINT8	Xhear_me : 1;		// TRUE if this voice is selected in the Hear dialog
	UNINT8	xxxxx	 : 7;		// unused bits
	INT8	Xmidisynth;			// type of MIDI synthesizer in used; see definitions below
#define synth_default	 0		// default synthesizer; use only basic MIDI features
#define synth_fb01		 1		// Yamaha FB01 with fractional pitch capabilities
#define synth_mono_pbend 2		// Use mono mode and pitch bend for fractional pitches
#define synth_silent	 3		// No MIDI output for this voice
	INT8	Xmonochans;			// If synth_mono_pbend, number of MIDI channels to use
	INT8	XvoiceOnPart;		// 0 for first voice on part, N-1 for further voices
	INT8	Xunused;			// unused byte in this node
} VINFO_node;
#define VINFO_sizes "1111111111111111111(17)111"



//								=== NOTE ===
// The NOTE struct defines all of the data that is stored about a single note or rest.
// You can look at the values of these fields in an existing Tilia file by using the
// "Internal Fields" menu item when you run Lime.
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is NOTE
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT8	Xmod7;				// 7*octave + notename; notename values are mod7_C,
								// mod7_D, mod7_E, ..., mod7_B.
								// An odd xmod7 value indicates the note is on
								// a staff-line in all clefs except TREBLEOCT
#define MAXMOD7 72				// highest value for xmod7
#define mod7_C	0
#define mod7_D	1
#define mod7_E	2
#define mod7_F	3
#define mod7_G	4
#define mod7_A	5
#define mod7_B	6

	UNINT8	Xnostem	  : 1;		// TRUE if stem for note is not to be plotted
	UNINT8	Xseparate : 1;		// TRUE to avoid combining rests together
	UNINT8	Xlgval	  : 4;		// lg of notated note length, uses these values:
#define LGB	  0					// xlgval for a breve note (a double-whole note)
#define LGW	  1					// xlgval for a whole note
#define LGH	  2					// xlgval for a half note
#define LGQ	  3					// xlgval for a quarter note
#define LGE	  4					// xlgval for a eighth note
#define LGS	  5					// xlgval for a sixteenth note
#define LGTS  6					// xlgval for a thirtysecond note
#define LGSF  7					// xlgval for a sixtyfourth note
#define LGOTE 8					// xlgval for a 128th note
#define MINLGVAL 0
#define MAXLGVAL 8
#define MAXTEMPOLGVAL 6			// thirtysecond note is fastest value for tempo marking
	UNINT8	Xbstem	  : 1;		// the stem goes between staves, and this is to go on
								// the "other" staff.  This is normally set on the lowest
								// notes in a stem-down chord (to move those lowest notes to
								// the staff below and extend the stem between the staves)
								// or on the highest notes in a stem-up chord (to move those
								// highest notes to the staff above and extend the stem between
								// the staves).  The staff below (for stem-down notes) or above
								// for stem-up notes) usually has only hidden rests.  See
								// comment in placecd.c for more information.
	UNINT8	Xtrembeam : 1;		// TRUE if the beam should be drawn to indicate a tremolo
								// between two different pitches.  This can only be used on
								// two notes which are beamed together and have a tuplet value
								// like "2 in the time of 16".  (This tuplet value indicates
								// that the two notes actually take the place of 16 notes.)
								// The notehead is changed to a longer note; in the case of
								// change to a whole note, the beams are drawn between the
								// noteheads; in the case of change to a quarter note or faster,
								// the beams are not drawn up to the stems.

	UNINT8	Xendvcd	  : 1;		// TRUE if last note in voice chord
	UNINT8	Xgliss	  : 1;		// glissando flag
	UNINT8	Xtie	  : 2;		// tie: NONE, UP, DOWN, UPORDOWN; indicates a tie from
								// this note to the next -- the ending note in the tie
								// does not have this field set
	UNINT8	Xtenu	  : 1;		// TRUE if note has tenuto mark
	UNINT8	Xstacc	  : 1;		// TRUE if note has staccato mark
	UNINT8	Xgrace	  : 1;		// TRUE if it is a grace note
	UNINT8	Xnoslash  : 1;		// if note is a grace note, TRUE if no slash on grace

	UNINT8	Xrest	  : 1;		// TRUE if it is a rest
	UNINT8	Xhider	  : 1;		// TRUE if rest is not to be plotted
	UNINT8	Xhidet	  : 1;		// TRUE if tuplet marks not to be plotted
	UNINT8	Xstring	  : 3;		// For guitar tablature notation: user-specified
								// string for the note; value ANYSTRING indicates
								// user did not specify a specific string.
#define ANYSTRING 0				// Use default algorithm to choose string for
								// tablature notation.
#define MAXSTRING 7				// largest string number allowed
	UNINT8	Xdots	  : 2;		// duration dots after a note
#define MAXDOTS 3				// triple-dotted note value is the maximum

#define DOUBLEFLAT 1
#define FLAT 2
#define NATURAL 3
#define SHARP 4
#define DOUBLESHARP 5
	UNINT8	Xacc	  : 3;		// accidental; one of the values defined above.
								// This accidental may not need to be plotted;
								// if it is implied by the key signature, or
								// by a preceding accidental in the same measure,
								// Xaccplot==FALSE can suppress plotting it.
								// Note that this field needs to be set even if
								// an SPACC node (special accidental symbol) is present;
								// xacc is needed in case SPACC is removed by user.
	UNINT8	Xaccplot  : 1;		// TRUE if accidental is to be plotted
	UNINT8	Xferm	  : 1;		// TRUE if there is a fermata
	UNINT8	Xdir	  : 2;		// stem direction  UP, DOWN, or UPORDOWN
	UNINT8	Xcbeam	  : 1;		// TRUE if note-beaming must not end here

	UNINT8	Xebeam	  : 1;		// TRUE if note-beaming must end here
	UNINT8	Xcsbeam	  : 1;		// TRUE if note-subbeaming must not end here
	UNINT8	Xesbeam	  : 1;		// TRUE if note-subbeaming must end here
	UNINT8	Xaccent	  : 2;		// 0 no accent, 1 small accent, 2 big accent
	UNINT8	Xmodvol	  : 1;		// for editing: indicates xvolume modified with louder/softer
	UNINT8	Xselected : 1;		// used by the editor to mark selected notes for
								// GROUP_SELECT when MODE==MUSIC_MODE
	UNINT8	Xcue	  : 1;		// TRUE if it is a cue note

	INT8	Xtupl;				// 1 for normal	  3 for triplets  see xintimeof field below

#define NH_ICON_MAX 8			// maximum NH value that is selectable from icon
								// xnotehead_to_char() maps to character numbers
#define NH_MAX	   15			// maximum value for xnotehead field
#define NHNORMAL	0			// normal notehead (3 types: whole, half, and filled)
#define NHPERC		1			// percussion notehead (2 types: whole&half, quarter&faster
#define NHOHARMONIC 2			// open harmonic notehead (1 type)
#define NHCHARMONIC 3			// closed harmonic notehead (1 type)
#define NHOLOZENGE	4			// open lozenge notehead (1 type)
#define NHCLOZENGE	5			// closed lozenge notehead (1 type)
#define NHNONE		6			// no notehead is plotted
#define NHPIANOROLL 7			// piano-roll notehead
#define NHSQUARE	8			// square notehead
	UNINT8	Xnotehead  : 4;		// notehead type; one of the values defined above
	UNINT8	Xslur	   : 2;		// slur: NONE, UP, DOWN, UPORDOWN; indicates a slur from
								// this note to the next -- the ending note in the slur
								// does not have this field set
#define TREMMAX 3
	UNINT8	Xtremolo   : 2;		// single-pitch tremolo:
								// 0, 1, 2 or 3 tremolo bars drawn on the note stem


	UNINT8	Xintimeof;			// used with xtupl field: "xtupl in the time of xintimeof";
								// examples: "3 in the time of 2" or "5 in the time of 7";
								// this field is shifted left by 2 to allow things like
								// "5 quarters in the time of 7 eighths" expressed by
								// "5 in the time of 3.5"
#define XINTIMEOF_SHIFT 2		// amount by which xintimeof values are shifted
#define XINTIMEOF_MULT 4		// amount by which xintimeof values are multipled
#define XINTIMEOF_MAX 60.0		// maximum value for xintimeof field (before shift)

	UNINT8	Xdashtie   : 1;		// TRUE if tie is to be dashed; used with Xtie field
	UNINT8	Xdashslur  : 1;		// TRUE if slur is to be dashed; used with Xslur field
	UNINT8	Xthumb	   : 1;		// TRUE for tablature note played by thumb
	UNINT8	Xmodp	   : 1;		// for notes: indicates xpitch modified with lower/higher;
								// obsolete for rests: indicates a pad-rest added for NIFFX alignment
	UNINT8	Xbendtie   : 1;		// TRUE if tie-with-pitch bend; draws a "bend" tie; in tab
								// a bend-up or bend-down symbol replaces tied-to note
	UNINT8	Xparen	   : 2;		// parentheses at notehead: NONE, LEFT, RIGHT, or BOTH
	UNINT8	Xbendslur  : 1;		// TRUE if slur-with-pitch bend; draws a "bend" slur; in tab
								// a bend-up or bend-down symbol replaces slurred-to note

	UNINT8	Xvolume;			// volume for playback, 0..200; user sees this
								// as a percentage of maximum 0..100 in steps of .5
	UNINT8	Xnachschlag : 1;	// this indicates grace after the beat (only set if also xgrace is set)
	UNINT8	Xcrawlslur : 1;		// TRUE to move slur at stem as close to notehead as possible
	UNINT8	Xvoice	   : 6;		// temporary data while Lime runs: voice number for node

	UNINT16 Xpitch;				// The audio pitch of the note.
								// This is an 8-bit notenumber (halfstep), followed
								// by an 8-bit fraction.
								// The 8-bit fraction is in half-cent units, and
								// takes on the range 0..200.
								// during MusicXML import: PACK_ID(partID,voiceID) for note
#define PITCHSHIFT		8
#define PITCHMASK		0xFF
#define MAXPITCH		0x7F00
#define PITCH_CENT		0x002	// A cent is 1/100th of a half-step
#define PITCH_HALFSTEP	0x100
#define HALF_CENT(xpch) (200 * ((xpch) >> PITCHSHIFT) + ((xpch) & PITCHMASK))
	INT32	Xlength;			// Playback duration: length in whole notes.
								// Format: 16 bit integer part, 16 bit fractional part.
	INT32	Xttn;				// Playback duration: time to next in whole notes.
								// Format: 16 bit integer part, 16 bit fractional part.
} NOTE_node;
#define NOTE_sizes "111(11411)(1121111)(11132)(31121)(1112111)1(422)1(1111121)1(116)244"



//								=== TIMESIG ===
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is TIMESIG
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT16	Xbpm;				// beats per measure (numerator)
	INT16	Xbeat;				// beat value (time signature denominator)
#define DEFAULT_TIMESIG 4		// default time signature is 4/4
	XYPOS	Xtimex;				// temporary data while Lime runs: x location of time sig
	XYPOS	Xtimeorigin;		// temporary data while Lime runs: relocation x of time sig
	INT8	Xtimetype;			// NORMAL_TIME, CUT_TIME, or COMMON_TIME
								// The xbpm and xbeat fields should be set, even
								// for cut time and common time.
#define NORMAL_TIME 0			// xtimetype value for normal time signature
#define CUT_TIME	1			// xtimetype value for cut time signature
#define COMMON_TIME 2			// xtimetype value for common time signature
	UNINT8	Xno_cwr : 1;		// TRUE to disable centered whole rests in this time signature
	UNINT8	notusedx : 7;
	INT8	Xunused[11];		// unused bytes in this node
} TIMESIG_node;
#define TIMESIG_sizes "1122xx1(17)"


//								=== CLEF ===
//
#define TREBLE			 0		// xclef value for treble clef
#define BASS			 1		// xclef value for bass clef
#define ALTO			 2		// xclef value for alto clef
#define TENOR			 3		// xclef value for tenor clef
#define SOPRANO			 4		// xclef value for soprano clef
#define TREBLEOCT		 5		// xclef value for treble-octave-down clef
#define BARITONE		 6		// xclef value for baritone clef (bass clef symbol)
#define PERCCLEF1		 7		// xclef value for single bar percussion clef
#define PERCCLEF2		 8		// xclef value for dual bar percussion clef
#define CYMBALCLEF		 9		// xclef value for cymbal clef
#define MEZZO_SOPRANO	10		// xclef value for soprano clef
#define C_BARITONE		11		// xclef value for baritone clef (c clef symbol)

typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is CLEF
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT8	Xclef;				// type of clef
	INT8	Xsmallclef;			// TRUE if it is small size (clef change)
	INT8	Xhideclef;			// TRUE to skip plotting the clef
	INT8	notUsedYet;
	XYPOS	Xclefx;				// temporary data while Lime runs: clef x location
	XYPOS	Xcleforigin;		// temporary data while Lime runs: clef relocation x
	INT8	Xunused[14];		// unused bytes in this node
} CLEF_node;
#define CLEF_sizes "111111xx"



//								=== KEYSIG ===
//
// The key signature may be any combination of sharps, flats, or naturals
// in any order.  The 'xkeynot[]' array specifies a list of pitches, and for
// each pitch, which accidental is to be plotted at that pitch.  The values
// of the 'xkeynot[]' array are defined as follows:
//	 0 flat on c	1 flat on d	   2 flat on e	  3 flat on f	 4 flat on g
//	 5 flat on a	6 flat on b	   7 sharp on c	  8 sharp on d	 9 sharp on e
//	10 sharp on f  11 sharp on g  12 sharp on a  13 sharp on b
// If 14 is added to these values, then naturals are plotted in the
// corresponding positions.  Note that sharps and flats on the same pitch don't
// always plot in the same octave in key signatures -- for example, the flat on
// f plots an octave lower than the sharp on f in treble clef.
//
// NOTE: It is important that an all-zero key signature node is C major.  This
// is used in several places if the piece has no key signature.
//

#define MAXKEY 14				// max number of sharps, flats, nats in key

typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is KEYSIG
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT8	Xkeycnt;			// number of sharp/flats/nats in key
	INT8	Xkeynot[MAXKEY-1];	// see comments above; last byte of array defined below
	INT8	XkeynotEND;			// last byte for Xkeynot[] array; this is defined
								// separately to defeat compiler's word alignment of
								// even-length byte arrays.
	INT8	Xtoplay;			// add this number of halfsteps to printed pitch
								// to get the playback pitch.  for example, this
								// is -2 for instruments in b-flat
	INT8	Xkey;				// type of key; order as defined below:
#define cmajor_key		0
#define gmajor_key		1
#define csharpmajor_key 7
#define fmajor_key		8
#define cflatmajor_key	14
#define aminor_key		15
#define eminor_key		16
#define asharpminor_key 22
#define dminor_key		23
#define aflatminor_key	29
#define MAJORKEY(kptr)			(xkey(kptr) < 15)
#define SHARPKEY(kptr)			((xkey(kptr) % 15) <= 7)
#define CNTKEY(kptr)			((SHARPKEY(kptr) ? xkey(kptr) : xkey(kptr) - 7) % 15)
#define XKEY_VALUE(mj,num,shp)	((mj ? 0 : 15) + num + ((shp || !num) ? 0 : 7))
#define KEY_NAT_OFFSET 14
#define MAJOR_SHARP_KEYS \
				QQ("\pC Major"), QQ("\pG Major"), QQ("\pD Major"), QQ("\pA Major"), QQ("\pE Major"), \
				QQ("\pB Major"), QQ("\pF Sharp Major"), QQ("\pC Sharp Major")
#define MINOR_SHARP_KEYS \
				QQ("\pA Minor"), QQ("\pE Minor"), QQ("\pB Minor"), QQ("\pF Sharp Minor"), QQ("\pC Sharp Minor"), \
				QQ("\pG Sharp Minor"), QQ("\pD Sharp Minor"), QQ("\pA Sharp Minor")
#define MAJOR_FLAT_KEYS \
				QQ("\pC Major"), QQ("\pF Major"), QQ("\pB Flat Major"), QQ("\pE Flat Major"), QQ("\pA Flat Major"), \
				QQ("\pD Flat Major"), QQ("\pG Flat Major"), QQ("\pC Flat Major")
#define MINOR_FLAT_KEYS \
				QQ("\pA Minor"), QQ("\pD Minor"), QQ("\pG Minor"), QQ("\pC Minor"), QQ("\pF Minor"), \
				QQ("\pB Flat Minor"), QQ("\pE Flat Minor"), QQ("\pA Flat Minor")


	UNINT8	Xcolorkey : 1;		// TRUE for automatic colorizing of key signature
	UNINT8	unused : 7;

	XYPOS	Xkeyx;				// temporary data while Lime runs: x location of key
	XYPOS	Xkeyorigin;			// temporary data while Lime runs: relocation x of key
} KEYSIG_node;
#define KEYSIG_sizes "1111111111111111111(17)xx"



//								=== TPARAM ===
// A TPARAM node specifies the location, style and font for text.
// The actual text string is contained in one or more TEXT nodes following
// the TPARAM node.
//
// If there are several lines of text, each line of text is implemented with
// its own TPARAM node followed by TEXT nodes.  There are no end-of-lines in
// TEXT nodes.
//
// If there is a font change within a line, then a new TPARAM node (with xadjacent
// set) is inserted at the proper place in the TEXT nodes; this TPARAM node is in
// turn followed by more TEXT nodes.
//
// TPARAM and TEXT nodes preced the NOTE node that the text is associated with.
// The location of the TEXT is given relative to the location that the
// associated NOTE gets during print formatting.
//

typedef struct {
	LINKS					// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;			// kind of node; value is TPARAM
	INT8	Xcontext;		// notation context; 0 all contexts, -1 not score

	XYPOS	Xtextx;			// x offset from notehead to first character; if xpagemargin or
							// xfooter then it is x offset from left margin; see also xcentered
	XYPOS	Xtexty;			// y offset to first character: if xfromstaff == TRUE,
							// the y offset is from the center staff line; if xfooter == TRUE,
							// the y offset is from the bottom of the page; otherwise
							// it is othe offset from the notehead y location
	INT16	Xtextfont;		// font number for text; certain font numbers remapped from
							// standard Mac numbers: TILIA_MARL, TILIA_TUFA, TILIA_SONATA,
							// TILIA_GENEVA, TILIA_TIMES, TILIA_SYMBOL; other font
							// number mappings are specified in FNAME nodes.
	UNINT8	Xcharset;		// Windows charset value for Windows foreign language support
	UNINT8	Xtextsize;		// font size (in points)
	Style	Xtextface;		// type face (italic, bold, underline etc).
							// "Style" is defined in <Quickdraw.h>.

	UNINT8	Xdrawmode	: 2;// drawing mode: ORMODE or BICMODE
#define ORMODE 0			// drawing mode -- Or
#define BICMODE 1			// drawing mode -- Bit Clear -- called "draw in white" for users
#define NODRAW 2			// drawing mode -- Do Not Draw (deleted xeocvar anno)
	UNINT8	Xkeep_hyphen : 1;
							// xkeep_hyphen is TRUE if trailing hyphens should stay
							// attached to text the item; FALSE indicates that the trailing
							// hyphen should be chopped off from the text and a centered
							// hyphen should be printed between this chord and the next.
	UNINT8	Xkeep_underline : 1;
							// xkeep_underline is TRUE if leading and trailing underline
							// characters should stay attached to text the item; FALSE
							// indicates that the underline characters should be chopped
							// off from the text and a line should be printed between this
							// chord's lyric and the next.
	UNINT8	Xno_size_adjust : 1;
							// Xno_size_adjust is TRUE if this text is the same size independent
							// of staff size changes; if Xno_size_adjust is FALSE, the text size
							// and location is adjusted (ie, on a cue staff in a different context);
							// see xtext_stsize
	UNINT8	Xpslayout	: 1;// TRUE if LineLayoutOn is to be used when printing this
							// text item in PostScript
	UNINT8	Xspec_underline : 1;
							// special underlining useful for text with superscripts:
							// Lime draws a single underline, no superscripted underline parts.
							// This used to be called "xchordsym" and also affected
							// transposability of text (now xclassify==cl_chordsymbol).
	UNINT8	Xbackslash	: 1;// TRUE if backslash expressions were used in typing this

	UNINT8 Xjustified	: 2;// LEFT_JUSTIFIED, CENTER_JUSTIFIED or RIGHT_JUSTIFIED.
							// Eg, if xjustified==CENTER_JUSTIFIED, the text is centered
							// around the location derived from (xtextx, xtexty).
	UNINT8 Xpagemargin	: 1;// This is presented as "horizontal lock" to users.
							// TRUE if the text should be placed relative to the
							// page margins, FALSE if xtextx is relative to notehead x.
							// The title of a piece usually has
							//		xpagemargin = TRUE
							//		xjustified = CENTERED.
							// This is overridden (implied set) if xfooter is set.

	UNINT8 Xboxed		: 1;// TRUE if the text should have a box drawn around it
							// (used for rehearsal marks)
	UNINT8 Xleavespace	: 1;// TRUE if horizontal space should be left for the text
							// when the music is formatted; this is used for lyrics.
							// Normal lyrics have the following values:
							//		xjustified = CENTERED
							//		xfromstaff = TRUE
							//		xfooter = FALSE
							//		xleavespace = TRUE
							//		xtextx = 0  (center around the notehead location)

	UNINT8 Xfromstaff	: 1;// This is presented as "vertical lock" to users.
							// TRUE if xtexty is the offset from the center staff line;
							// FALSE if xtexty is the offset from the notehead y
							// This is overridden if xfooter is set.
	UNINT8 Xadjacent	: 1;// TRUE if the text is a font-changed continuation of the
							// previous text; only the font, size, and style information
							// fields of the TPARAM node are used if xadjacent is set.
							// This is never used together with xeocvar, since
							// xadjacent is only generated by user annotations with
							// font changes, and xremove cannot be used to hide user annos.
	UNINT8 Xeachsys		: 1;// TRUE if this was inserted using Duplicate At Each System.
							// This has no special meaning; it is only used by the editor
							// if the user uses Clear At Each System. This field is
							// provided to allow the user to undo an accident.

	UNINT8 Xfing		: 1;// This is presented as style "fingering" or "stack vertical" to the users.
							// It is used to print fingering information in a vertical stack.
#define MAXFING 10			// max number of numerals in fingering TEXT node (5 is normal max)
	UNINT8 Xeverypart	: 1;// TRUE if this was inserted using Duplicate In Every Part.
							// If this bit is set, the annotation will only print if
							// this is the score and xcontext==SCORE_NCONTEXT, or if
							// this is not the score and xcontext==NOT_IN_SCORE_NCONTEXT
							// and this is the top staff in the current notation context; it is
							// also used by the editor if the user uses Clear In Every Part.
							// See anno_everypart_dup() in annocopy.c.
	UNINT8 Xhidden		: 1;// TRUE if the text should not be printed on paper; on the
							// screen the text appears with a dotted underline to
							// indicate hidden text.  This is useful for dynamic markings
							// or other text which is not to be printed. This should
							// not be confused with xeocvar or xdrawmode==BICMODE.
#define HIDDEN_DESC 30		// hidden text: dotted underline distance below (tenth pt)
#define HIDDEN_GAP 2		// hidden text: dotted underlines gap length
#define HIDDEN_DASH 1		// hidden text: dotted underlines dash length
#define HIDDEN_WIDTH 1		// hidden text: dotted underlines line width
#define OFFPAGE_DASH 5		// text moved from offpage: dashed indicator length
#define OFFPAGE_GAP 1		// text moved from offpage: dashed indicator length
	UNINT8 Xdefault		: 1;// TRUE if this anno serves as default for xclassify value
	UNINT8 Xeachpage	: 1;// TRUE if this was inserted using Duplicate On Each Page.
							// This has no special meaning; it is only used by the editor
							// if the user uses Clear On Each Page. This field is
							// provided to allow the user to undo an accident.
	UNINT8 Xunicode		: 1;// Annotation is Unicode (reserved field)
	UNINT8 Xfooter		: 1;// TRUE if position is relatvie to bottom left of page; this value
							// overrides xfromstaff and xpagemargin
	UNINT8 Xignore		: 1;// TRUE if Lime should not interpret anno

	UNINT8 Xtext_stsize : 4;// original staff size, in points; used with xno_size_adjust field
	UNINT8 Xtext_color	: 4;// color index into color table

	UNINT8 Xeocord;			// ordinal of this variety; used with xeocvar
	UNINT8 Xeocvar;			// zero if this is a regular annotation; else it is the variety
							// of Line graphic being modified by this TPARAM/TEXT.  This
							// is for Allow Any Option-click.
	INT8 Xsuper;			// amount to superscript (positive) or subscript (negative)
	INT8 Xexpand;			// amount to expand (move right) or condense (move left)
	INT8 Xclassify;			// classify type of text
	
	UNINT8 Xtparamnused1;
	//						// If non-zero, following two fields override xfooter, xfromstaff and xpagemargin
	//						// (currently only set and used internally, but will be used in next version)
	UNINT8 Xrelativey	: 4;// what y positioning is relative to 
	UNINT8 Xrelativex	: 4;// what x positioning is relative to	
	
	INT16 Xfontd;			// for Lime internal use: font description in MusicXML export
	INT16 Xstbl;			// for Lime internal use: string table index in MUSICXML export;
							// PACK_ID(partID,voiceID) for string for MusicXML import
} TPARAM_node;
#define TPARAM_sizes "11xx2111(2111111)(2111111)(11111111)(44)111111(44)22"

// Enumeration for the Xjustified field
//
enum{
	LEFT_JUSTIFIED			= 0,
	CENTERED				= 1,	// Centered but not moveable
	RIGHT_JUSTIFIED			= 2,
	CENTER_JUSTIFIED		= 3		// Centered and moveable	
};

// Enumeration for the Xrelative fields
//
enum{
	NIL_RELATIVE			= 0,	// unspecified (should never be used)
	NOTE_RELATIVE			= 1,	// +right/up,-left/down
	PAGE_RELATIVE			= 2,	// from bottom/left	+right/up,-left/down
	PAGE_LEFT_RELATIVE		= 2,	// from left		+right,(-left!)
	PAGE_BOTTOM_RELATIVE	= 2,	// from bottom		+up,(-down!)
	PAGE_RIGHT_RELATIVE		= 3,	// from right		+right,-left
	PAGE_TOP_RELATIVE		= 3,	// from top			-down,(+up!)
	PAGE_CENTER_RELATIVE	= 4,	// from the middle point in the page	+right/up,-left/down
	STAFF_RELATIVE			= 5,	// y only: middle +up/-down
	STAFF_LEFT_RELATIVE		= 5,	// x only: from staff left	+right,-left 
	STAFF_RIGHT_RELATIVE	= 6,	// x only: from staff right	+right,-left
	MARK_RELATIVE			= 6,	// y only: relative to last expression mark placed automatically
	MARK_ABOVE_RELATIVE		= 7,	// y only: relative to last expression mark if placed above
	MARK_BELOW_RELATIVE		= 8,	// y only: relative to last expression mark if placed below
	MARK_LEFT_RELATIVE		= 7,	// x only: relative to last mark (accidental) on the left
	MARK_RIGHT_RELATIVE		= 8,	// x only: relative to last mark (e.g. dot)   on the right [this may not be possible
	BAR_RELATIVE			= 9,	// x only: +right,-left
	BAR_LEFT_RELATIVE		= 9,	// x only: +right,-left
	BAR_RIGHT_RELATIVE		= 10,	// x only: +right,-left	[this may not be possible]
	
	// Masks identifying note and/or staff related relativity.
	//
	NOTE_RELATIVExMASK		= 1 | (1<<NOTE_RELATIVE) | (1<<MARK_LEFT_RELATIVE) | (1<<MARK_RIGHT_RELATIVE),
	NOTE_RELATIVEyMASK		= 1 | (1<<NOTE_RELATIVE) | (1<<MARK_RELATIVE) | (1<<MARK_ABOVE_RELATIVE) | (1<<MARK_BELOW_RELATIVE),
	NOTE_STAFF_RELATIVEyMASK= 1 | NOTE_RELATIVEyMASK | (1<<STAFF_RELATIVE)
};

// Enumeration for Xclassify field.	  JA20120228 Moved here from Lnktilia.h
//
enum {
	cl_notTextAnnotation	= -128,	// Used internally
	cl_notClassified		= 0,
	cl_other				= 1,
	cl_title				= 2,
	cl_composer				= 3,
	cl_rehearsal			= 4,
	cl_lyric				= 5,
	cl_tempo_marking		= 6,
	cl_dynamic				= 7,
	cl_fingering			= 8,
	cl_ped					= 9,
	cl_fretgrid				= 10,
	cl_chordsymbol			= 11,
	cl_midi					= 12,
	cl_label				= 13,
	cl_articulation			= 14,
	cl_breath				= 15,
	cl_ornament				= 16,
	cl_repeat				= 17,
	cl_copyright			= 18,
	cl_bowing				= 19, cl_technical = cl_bowing,
	cl_pause				= 20,
	cl_mood					= 21,
	cl_tempo_alteration		= 22,
	cl_poet					= 23,
	cl_subtitle				= 24,
	
	cl_TOTAL
};




//								=== TEXT ===
// Each TEXT node holds at most MAXTEXT characters.  If a text string is longer
// than this, it is represented by successive TEXT nodes.
//
// TEXT nodes do no contain end-of-lines or font changes; TPARAM nodes are inserted
// between TEXT nodes in this case.  See the discussion of TPARAM nodes above.
//
#define MAXTEXT 22				// max number of characters represented in a TEXT node,
								// including space for the terminating '\0'

typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is TEXT
	INT8	Xcontext;			// defines the notation contexts this node applies to

	char	Xtext[MAXTEXT];		// Text character string.  The string is terminated
								// by '\0', even if the text continues in a following
								// TEXT node.
} TEXT_node;
#define TEXT_sizes "111111111111111111111111"



//								=== LINE ===
// Lines or curves.
//
// During formatting, a PDATA node gets inserted after the LINE node.
// The PDATA node acts as an extension of the LINE node (as well as the
// TPARAM node).
//
typedef struct {
	LINKS					// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;			// kind of node; value is LINE
	INT8	Xcontext;		// defines the notation contexts this node applies to

	XYPOS	Xstartx;		// x offset from notehead to start of line/curve
	XYPOS	Xstarty;		// y offset from notehead to start of line/curve
	XYPOS	Xendx;			// x offset from start of line to end of line
	XYPOS	Xendy;			// y offset from start of line to end of line

	UNINT8	Xdashlth : 4;	// dashed lines and curves: length of dash segments in
							// dots; zero (NODASH defined below) indicates solid
	UNINT8	Xgaplth : 4;	// dashed lines and curves: length between gap segments
							// in dots; zero (NODASH defined below) indicates solid
#define MAX_DASHLTH 15		// maximum value for xdashlth
#define MAX_GAPLTH 15		// maximum value for xgaplth
#define NODASH 0			// indicates solid line; in subroutine calls,
							// used for both xgaplth and xdashlth

	UNINT8	Xline_ord : 7;	// if xline_eocvar!=0, xline_ord is ordinal of this variety
							// if xline_eocvar==0, xline_ord nonzero is defined below:
	// Resizing lines/hairpins/8va have two LINE nodes; both LINE nodes have xline_eocvar==0.
	// The LINE node anchoring the beginning of the line has xline_ord==BEGLINE.
	// The LINE node anchoring the end of the line has xline_ord==ENDLINE.
	// The two LINE nodes must have matching xstarty and xendy fields, but its xstartx and xendx
	// fields may be different.
	//
	// A LINE node with xline_ord==MIDLINE (and xline_eocvar==0) is a Lime-generated node,
	// without any permanent information.  These are created and destroyed each time Lime
	// reformats the page; they aid in drawing resizing lines that span multiple system
	// breaks.
#define ENDLINE 1			// xline_ord value forLINE node anchoring the end-of-line
#define MIDLINE 2			// xline_ord value for temporary LINE node in middle of multi-system line
#define BEGLINE 3			// xline_ord value forLINE node anchoring the start-of-line
#define is_RESIZE(p)	(xkind(p) == LINE && !xline_eocvar(p) && xline_ord(p))
#define is_ENDLINE(p)	(is_RESIZE(p) && xline_ord(p) == ENDLINE)
#define is_MIDLINE(p)	(is_RESIZE(p) && xline_ord(p) == MIDLINE)
#define is_BEGLINE(p)	(is_RESIZE(p) && xline_ord(p) == BEGLINE)
#define set_ENDLINE(p)	(xline_eocvar(p) = 0, xline_ord(p) = ENDLINE)
#define set_MIDLINE(p)	(xline_eocvar(p) = 0, xline_ord(p) = MIDLINE)
#define set_BEGLINE(p)	(xline_eocvar(p) = 0, xline_ord(p) = BEGLINE)
	UNINT8	Xcurve : 1;		// TRUE for curves, FALSE for lines.  If xcurve is TRUE,
							// then (xstartx, xstarty) give the start of the curve;
							//		(xendx, xendy) give the end of the curve
							//		(xc1x, xc1y) give the first control point
							//		(xc2x, xc2y) give the second control point.
							// If xcurvex is FALSE, then xc1x, xc1y, xc2x, xc2y
							// are corners of a clip rectangle for the line.

	XYPOS	Xc1x, Xc1y;		// offset from start of curve/line to first control point
	XYPOS	Xc2x, Xc2y;		// offset from start of curve/line to second control point

	UNINT8	Xfattermid : 1; // TRUE if the curve becomes fatter in the middle
	UNINT8	Xline_fromstaff : 1;
							// like xfromstaff for TPARAM: TRUE if
							// xstarty is the offset from the center staff line;
							// FALSE if xstarty is the offset from the
							// notehead y.
	UNINT8	Xline_eachsys : 1;
							// TRUE if this was inserted using Duplicate At Each System.
							// This has no special meaning; it is only used by the editor
							// if the user uses Clear At Each System. This field is
							// provided to allow the user to undo an accident.
	UNINT8	Xline_eachpage : 1;
							// TRUE if this was inserted using Duplicate On Each Page.
							// This has no special meaning; it is only used by the editor
							// if the user uses Clear On Each Page. This field is
							// provided to allow the user to undo an accident.
	UNINT8	Xline_everypart : 1;
							// TRUE if this was inserted using Duplicate In Every Part.
							// If this bit is set, the annotation will only print if
							// this is the top staff in the current notation context; it is
							// also used by the editor if the user uses Clear In Every Part.
	UNINT8	Xline_drawmode : 2;
							// like xdrawmode for LINE: ORMODE or BICMODE
	UNINT8	Xline_pagemargin : 1;
							// like xpagemargin for TPARAM: TRUE if the line
							// should be placed relative to the page margins,
							// FALSE if xstartx is relative to notehead x.

	UNINT8 Xlinewidth;		// width of line in tenth point units
#define REGLINE 10			// default used for bar lines, etc
#define THINLINE 2			// default used for stems, staff lines, etc
#define FATLINE 25			// default used in beaming notes
#define GRACE_FATLINE 15	// default used in beaming grace notes
#define GROUP_SELECT_LW 5	// default line width used for sample_line in
							// GROUP_SELECT; must be different from REGLINE,
							// THINLINE, and FATLINE so that none of the
							// width buttons in the annotation controls are
							// selected
	UNINT8 Xmetaline : 4;	// nonzero if this LINE node represents a hairpin, tuplet
							// bracket, or a bracket over notes.
	UNINT8 Xline_color : 4; // color index into color table

	UNINT8 Xxxline_resize : 1; // obsolete
	UNINT8 Xline_eocvar : 7;// zero if this is a regular annotation; else it is the variety
							// of graphic being modified by this LINE node.  This is for
							// Extended Option-click.

	UNINT8 Xlineused1;
	//						// If non-zero, following two fields override Xline_fromstaff and Xline_pagemargin
	//						// (currently only set and used internally, but will be used in next version)
	UNINT8 Xline_rely	:4;	// what y positioning is relative to 
	UNINT8 Xline_relx	:4;	// what x positioning is relative to	
	
	INT16  Xline_stbl;		// for Lime internal use: PACK_ID(partID,voiceID) for string for MusicXML import
} LINE_node;
#define LINE_sizes "11xxxx(44)(71)xxxx(1111121)1(44)(17)1(44)2"

enum {
	STRAIGHT_LINE		= 0,
	HAIRPIN				= 1,	// xmetaline: this LINE node specifies one line of a hairpin
	TUPLET_BRACKET		= 2,	// xmetaline: this LINE node specifies how to place a tuplet bracket
	PLAIN_BRACKET		= 3,	// xmetaline: this LINE node specifies how to place a plain bracket
	RIGHT_BRACKET		= 4,	// xmetaline: this LINE node specifies right-only bracket (e.g, 8va line)
	LEFT_BRACKET		= 5,	// xmetaline: this LINE node specifies left-only  bracket
	RECTANGLE			= 6,	// xmetaline: this LINE node specifies a full rectangle
	RIGHT_RECTANGLE		= 7,	// xmetaline: this LINE node specifies right-only  rectangle
	LEFT_RECTANGLE		= 8,	// xmetaline: this LINE node specifies left-only   rectangle
	BOTTOM_RECTANGLE	= 9,	// xmetaline: this LINE node specifies bottom-only rectangle
	TOP_RECTANGLE		= 10,	// xmetaline: this LINE node specifies top-only    rectangle
	tot_METALINE
};



//							   === xxxFNAME ===
// Obsolete node to specify a font number and the associated font name.
// This definition is kept only for converting old Lime pieces.
//
#define xxxMAXFNAME 16			// obsolete max number of characters in xxxFNAME node

typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is xxxFNAME
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT16	XXXfontnum;			// obsolete: font number, as used in TPARAM nodes
	char	XXXfontnam[xxxMAXFNAME];// obsolete: name for the font, must be terminated with '\0'
} xxxFNAME_node;
#define xxxFNAME_sizes "1121111111111111111"  // for obsolete node





//							   === xxxNIFFX ===
// Obsolote NIFFX nodes were created by the Import NIFF menu option.  They stored x-location
// from the NIFF representation.
//

typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is NIFFX
	INT8	Xcontext;			// defines the notation contexts this node applies to

	XYPOS	XXXniffx;			// obsolote: NIFF x location for the note's timeslice; this is in Tilia
								// units; value is after offset/skew correction for rotated
								// scan image.
	UNINT8	xxxniff_rhythm : 1; // obslete: TRUE to mark probable rhythm errors
	UNINT8	XXXnotused : 7;
	UNINT8	XXXnotusedyet;
	XYPOS	XXXniffy;			// obsolete: NIFF staff y location; this uses Tilia units except that
								// origin is at top; value is after offset/skew correction for
								// rotated scan image.
} xxxNIFFX_node;
#define xxxNIFFX_sizes "11x(17)1x"


//								=== BAR ===
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is BAR
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT8	Xdouble;			// type of bar line
#define REGBAR 0				// single bar
#define DOUBLEBAR 1				// double bar
#define ENDBAR 2				// end of piece double bar: second bar fatter
#define BEGBAR 3				// first bar fatter
	INT8	Xxxbetween;			// obsolete: TRUE for bar between staff, staff+1 (see BARSEP node)
	INT8	Xleftdots;			// TRUE if repeat dots on left of bar
	INT8	Xrightdots;			// TRUE if repeat dots on right of bar
	INT8	Xunnumbered;		// TRUE if measure is not counted in the
								// measure numbering

	UNINT8	Xwings : 1;			// TRUE to use thicker repeat signs with wings
	UNINT8	Xrestart : 1;		// TRUE if measure numbering restarts at xrestart_num
	UNINT8	Xhiden : 1;			// TRUE avoids printing measure number at start of sys
	UNINT8	Xchant : 1;			// TRUE for chant bar line
	UNINT8	notusedyet2 : 4;

	XYPOS	Xbarx;				// temporary data while Lime runs: bar x location
	XYPOS	Xbarorigin;			// temporary data while Lime runs: bar relocation x
	INT8	Xnotstaff;			// TRUE means: don't draw bar on the staff itself
								// (from top to bottom staff line assuming a five-line
								// staff).  Together with BARSEP nodes, this
								// controls where the bar line is drawn.
	INT8	Xdashed;			// TRUE if the bar line is dashed
	XYPOS	Xscrbarx;			// temporary data while Lime runs: screen x location
	INT16	Xrestart_num;		// if xrestart is TRUE, then restart bar numbering with
								// this value.
	INT8	Xgoto_type;			// for hear option: see values defined below
#define NO_GOTO 0				// xgoto_type value: continue playing regularly
#define FIRST_ENDING 1			// xgoto_type value: start of first ending
#define SECOND_ENDING 2			// xgoto_type value: start of second ending
#define FIRST_TIME 3			// xgoto_type value: first time go to xgoto_meas
#define SECOND_TIME 4			// xgoto_type value: second time go to xgoto_meas
#define THIRD_TIME 5			// xgoto_type value: third time go to xgoto_meas
	INT8	Xhear_count;		// temporary data while Lime runs: count how often Hear
								// option played through this barline
	INT16	Xgoto_bar;			// for hear option: continue at this measure; used if
								// xhear_count and xgoto_type have certain values
	INT8	Xunused[2];			// unused bytes in this node
} BAR_node;
#define BAR_sizes "1111111(11114)xx11x2112"

enum{
	XBARX_NOT_SET	= UNDEFXY	// initial (undefined) value for Xbarx and Xscrbarx
};



//								  === BARSEP ===
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is BARSEP
	INT8	Xcontext;			// defines the notation contexts this node applies to

	// This node exists before the same note as a BAR node, if the  bar is to be separated
	// (not drawn between staff, staff+1).  This is the complement of the BAR node's obsolete xbetween()
	// field; this function now has its own node, so that it can depend on notation contexts.

	INT8	notUsed;
} BARSEP_node;
#define BARSEP_sizes "111"




//								   === ENDSYST ===
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is ENDSYST
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT8	Xendpage;			// TRUE if it is end of page
	INT8	Xendpiece;			// TRUE if it is end of piece
	INT8	Xstartpiece;		// TRUE if it is start of piece

	UNINT8	Xsys_slurup : 1;	// TRUE if there is a up-slur off end of system; this
								// field is set by the formatter, so that this
								// information is available for the next system/page.
	UNINT8	Xsys_slurdown : 1;	// TRUE if there is a down-slur off end of system; this
								// field is set by the formatter, so that this
								// information is available for the next system/page.
	UNINT8	Xsys_beamup : 1;	// TRUE if there is a up-stem beam off end of system; this
								// field is set by the formatter, so that this
								// information is available for the next system/page.
	UNINT8	Xsys_beamdown : 1;	// TRUE if there is a down-stem beam off end of system; this
								// field is set by the formatter, so that this
								// information is available for the next system/page.
	UNINT8	Xbracket : 2;		// controls bracket drawing at start of system.  If xstartpiece
								// is set on this ENDSYST node, this field is ignored and
								// the xfirst_bracket field in the first PRINT node is used
								// instead; this is because the first ENDSYST node is shared
								// among all notation contexts, but bracket controls should not be.
								// The following are values for xbracket and xfirst_bracket:
#define BRACKET_BETWEEN 0		// bracket on staff and to next staff
#define BRACKET_ON_STAFF 1		// bracket is drawn on staff
#define NO_BRACKET 2			// no bracket drawn for this staff
	UNINT8	Xbrace : 2;			// controls curly brace drawing at start of system. If xstartpiece
								// is set on this ENDSYST node, this field is ignored and
								// the xfirst_brace field in the first PRINT node is used
								// instead; this is because the first ENDSYST node is shared
								// among all notation contexts, but bracket controls should not be.
								// The following are values for xbrace and xfirst_brace:
#define NO_BRACE 0				// no curly brace drawn for this staff
#define BRACE_BETWEEN 1			// brace between this staff and next staff

	INT32	Xttstart;			// This field is used only if xstartpiece==TRUE.
								// It indicates the time that should elapse
								// before the voice starts playing (analogous to
								// the xttn field in NOTE nodes).  This allows
								// one voice to start with grace notes while
								// another voice has an implied rest.

#define MAX_SYS_TIE 5			// Maximum number of ties over the end of the system
								// for one voice; there are this many bowed up and
								// again this many bowed down.
#define NO_SYS_TIE 0			// indicates no tie for xsys_tie fields
#define DASHED_SYS_TIE_BIT 0x80 // bit in sys_tieup[] and sys_tiedown[] for dashed
#define MOD7_SYS_TIE_FIELD 0x7f // field in sys_typeup[] and sys_tiedowm[] for mod7
	INT8	Xsys_tieup[MAX_SYS_TIE];
								// xmod7 values for up-ties off end of system;
								// a NO_SYS_TIE value indicates no tie.  These fields
								// are set and used by the formatter.
	INT8	Xsys_tiedown[MAX_SYS_TIE];
								// xmod7 values for down-ties off end of system;
								// a NO_SYS_TIE value indicates no tie.  These fields
								// are set and used by the formatter.
	INT8	Xsys_voice;			// voice number which this ENDSYST node belongs to; this
								// is set by findpage() in page.c, and it is used in
								// param.c when looking for PARAM nodes before this page.

	UNINT8	Xsys_dash_slurup	: 1;	// TRUE if up-slur is dashed
	UNINT8	Xsys_dash_slurdown	: 1;	// TRUE if down-slur is dashed
	UNINT8	Xsys_crawl_slurup	: 1;	// TRUE if up-slur is Near Notehead
	UNINT8	Xsys_crawl_slurdown : 1;	// TRUE if down-slur is Near Notehead
	UNINT8	Xsys_bend_slurup	: 1;	// TRUE if up-slur is with pitch bend
	UNINT8	Xsys_bend_slurdown	: 1;	// TRUE if down-slur is with pitch bend
	UNINT8	Xavail				: 2; // these bits are not used

	INT8	Xunused[2];			// unused bytes in this node
} ENDSYST_node;
#define ENDSYST_sizes "11111(111122)411111111111(1111112)"


//								=== PRINT ===
// Tilia PRINT nodes are generated by the "Parts and Voices..." menu item in Lime.
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is PRINT
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT16	Xsstaff;			// staff number in system; NOSTAFF indicates Don't Print (if
								// xlist!=NOLIST) or Deactivated Voice (if xlist==NOLIST).
	INT16	Xprintdir;			// UP, DOWN or UPORDOWN for print direction
								// of the voice on the staff

	INT16	Xslines;			// number of staff lines (usually 5)
#define PIANOTAB_RH_SLINES 7	// number of staff lines in piano tablature right hand
#define PIANOTAB_LH_SLINES 8	// number of staff lines in piano tablature left hand
#define MAX_SLINES 12			// max number of staff lines in non-tablature staff

	INT16	Xlhrh;				// temporary while Lime runs: indicates a voice switching
								// between staves, potentially beaming between the staves
#define NOLHRH 0				// xlhrh value: no switch
#define LH 1					// xlhrh value: switch to lower staff
#define RH 2					// xlhrh value: switch to higher staff

	INT8	Xxxforcues;			// OUTDATED: TRUE if staff is cue-size staff

	UNINT8	Xfirst_bracket : 2; // controls bracket drawing at start of system for the first
								// system in the piece; if this is not the first PRINT node on
								// the voice, this field is ignored.  See comments on xbracket
								// field definition for the ENDSYST node.
	UNINT8	Xfirst_brace : 2;	// controls brace drawing at start of system for the first
								// system in the piece; if this is not the first PRINT node on
								// the voice, this field is ignored.  See comments on xbrace
								// field definition for the ENDSYST node.
	UNINT8	Xtablature : 2;		// nonzero if tablature notation is to be used;
								// normally the user also selects 6 staff lines.
								// 1 means stemless tablature, 2 is tab with stems,
								// 3 is piano tablature
#define NOT_TAB 0				// xtablature for normal notation
#define TAB_STEM 1				// xtablature for guitar tablature with stems
#define TAB_NOSTEM 2			// xtablature for stemless guitar tablature
#define TAB_PIANO 3				// xtablature for piano tablature
	UNINT8	unusedx : 2;

	INT16	Xstaffsize;			// points between staff lines (times STD_SF)

	UNINT8	Xpartstaff : 4;		// staff number within part (if xsstaff != NOSTAFF), 0 is first staff in part
	UNINT8	Xpartstaves : 4;	// number of staves for part (0 means xpartstaff not valid)
	INT8	Xlist;				// for Lime internal use: list entry in Voices... dialog;
								// value is 1..N, NOLIST means "deactivated voice".
	INT8  Xlayer;				// nonzero: layered voice (print using separate internal staff representation);
								// special value ALTLYR automatically set by altlayer().

	INT8  Xunused[5];			// unused bytes in this node
} PRINT_node;
#define PRINT_sizes "1122221(2222)2(44)11"



//								=== SDRAG ===
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is SDRAG
	INT8	Xcontext;			// defines the notation contexts this node applies to

	XYPOS	Xstaffdrag;			// Displace the current staff from its normal
								// y position by this amount.  This field is set
								// when an individual staff is draged.
	INT16	Xsdrag_sstaff;		// staff number within system for which staff drag was made for;
								// avoids staff jumping if voices switch between staves
	INT8	Xunused[18];		// unused bytes in this node
} SDRAG_node;
#define SDRAG_sizes "11x2"



//								=== BEAMING ===
//
#define MAXBVALUE 4				// maximum number of beaming values per voice
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is BEAMING
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT32	Xsbvalue;			// Subbeaming value (16 bit whole-note, 16 bit fractional).
								// This value is used to determine sub-beaming boundaries.
	INT32	Xbvalue[MAXBVALUE]; // Beaming value (16 bit whole-note, 16 bit fractional).
								// A list of beat values is given for each voice.
								// If less than MAXBVALUE values are used, the list of values
								// is terminated by a zero value.
								// The list is repeated if necessary for long measures.
								// These values are used to determine beaming boundaries.

	INT8	Xunused[2];			// unused bytes in this node
} BEAMING_node;
#define BEAMING_sizes "1144444"


//								=== SPACE ===
//
// This node is supported only for backward compatability; xleavespace annotations
// and chord drag make this an unnecessary node type.
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is SPACE
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT16	Xspace;				// Make sure there is at least this much blank space
								// before the note.  Supported for backward compatability;
								// replaced by xleavespace and chord drag.
	INT8	Xunused[20];		// unused bytes in this node
} SPACE_node;
#define SPACE_sizes "112"



//								=== CMOVE ===
// This is used to move one chord to a new x location.  It does not affect any
// other chords.
//
// It can also be used to move rests to a new y location.
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is CMOVE
	INT8	Xcontext;			// defines the notation contexts this node applies to

	XYPOS	Xchordmove;			// Displace the chord from its normal x position
								// this amount; do not relocate any other chords.
								// This field is set after all objects have been
								// placed, and an individual chord is moved.  This
								// can be used to skew chord placement on different
								// staves: chords at the same time may not plot
								// at the same x location.
	INT8	Xunused[20];		// unused bytes in this node
} CMOVE_node;
#define CMOVE_sizes "11x"



//								=== AMOVE ===
// This is used to move one note's accidentals to a new x location.
// It does not affect any other notes or their accidentals.
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is AMOVE
	INT8	Xcontext;			// defines the notation contexts this node applies to

	XYPOS	Xaccmove;			// Displace the note's accidental from its normal
								// x position this amount.
	INT8	Xunused[20];		// unused bytes in this node
} AMOVE_node;
#define AMOVE_sizes "11x"






//								=== RMOVE ===
// This is used to move a rest to a new y location.
// It does not affect any other notes or rests.
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is RMOVE
	INT8	Xcontext;			// defines the notation contexts this node applies to

	XYPOS	Xrestmove;			// Displace the rest from its normal y position by
								// this number of staff spaces.
#define MAX_RESTMOVE 20			// move no more than 20 staff spaces up or down
	INT8	Xunused[20];		// unused bytes in this node
} RMOVE_node;
#define RMOVE_sizes "11x"



//								=== ZONE ===
// ZONE nodes bound relocation zones for notes.  They specify an x location (as a
// permil of system width) at which the associated chord should be located; the
// chord is moved to this location, and the chords before and after this chord are
// relocated appropriately.  This allows the user control over general note density
// in different parts of the system; it also allows the user to line up measure
// bars on different systems.
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is CMOVE
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT16	Xzone_xpermil;		// Displace the chord from its normal x position
								// to this position; relocate chords before and after
								// this chord appropriately. This is used to make
								// bar lines on different systems line up, or to make
								// a measure wider or narrowed than the default.
								// This quantity is given in permil (tenth percents)
								// of the total system width.
	INT32	Xzone_wholec;		// Whole note count at start of zone.
	INT32	Xzone_sys_wholec;	// Whole note length of whole system; this is kept to
								// help detect ZONE nodes which are out of date due to
								// systems being split or combined, or due to measures
								// being deleted, etc. If the length of the system does
								// not match xzone_sys_wholec, then the ZONE node is out
								// of date and will be removed.
	INT8	Xzone_bar;			// TRUE if we are specifying the previous bar location
	INT8	Xunused[11];		// unused bytes in this node
} ZONE_node;
#define ZONE_sizes "112441"


//								=== PARAM ===
// The "Parameters" menu option in Lime creates hidden text annotations, which specify
// changes to various music formatting parameters.  When Lime formats a page of music for
// display,
//		(1) it deletes all existing PARAM nodes,
//		(2) it interprets hidden text annotations and inserts corresponding PARAM nodes,
//		(3) it uses these PARAM nodes (for efficient look-up) in later formatting passes.
// PARAM nodes are recreated each time the page is formatted; thus if the user deletes, moves,
// or modifies the corresponding hdden text annotation, the PARAM nodes will reflect the change
// the next time the page is reformatted.  PARAM nodes are stored on disk in the Tilia file.
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is PARAM
	INT8	Xcontext;			// defines the notation contexts this node applies to

	DefTiliaPtr Xprev_param;	// temporary pointer while Lime runs: points to the previous
								// PARAM node on this voice.  This reverse linked list
								// is used to find PARAM nodes on previous pages quickly.
	INT16	Xparam_index;		// The parameter number which is being modified by
								// the param node; as described above, use
								// Option-Parameters in Lime to see these numbers

	UNINT8	Xparam_small : 1;	// TRUE if the small-size (cue or grace note size) version
								// of the parameter is to be modified; FALSE if the normal
								// size version of the parameter is to be modified.
	UNINT8	unusedbits : 7;		// bits available for future use

	INT8	unusedbyte;			// byte available for future use
	PARAM_VALUE Xparam_value;	// Either a specification for a new character mapping,
								// an integer value, or a floating value: depends on type
								// of parameter indicated by Xparam_index.
#define PVAL_TO_PTR 18			// Bytes offset between Xparam_value and Xnext field.
								// NOTE: This PVAL_TO_PTR value must be defined
								// properly for Lime's internal copy_param() function.
#define PVAL_TO_PINDEX 4		// Bytes offset between Xparam_value and Xparam_index.
	INT16	Xparam_stsize;		// Size of staff at time of parameter specification.
	INT8	Xunused[2];			// unused bytes in this node
} PARAM_node;
#define PARAM_sizes "1142(17)1+2"	/* + indicates PARAM_VALUE union, 10 bytes */


//								=== SPACC ===
//
// If a note has one or more special accidentals, these are stored in the
// SPACC node.
//

#define MAXSPACC 15				// max number of accidental symbols before a note

typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is SPACC
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT8	Xnumspacc;			// number of accidentals before note; values range
								// from 1 to MAXSPACC
	INT8	Xspacc[MAXSPACC];	// list of accidentals for note; value between
								// DOUBLEFLAT and DOUBLESHARP indicates standard symbols,
								// value of SPECIALACC+N-1 is special accidental
								// symbol N in Parameters.
#define SPECIALACC 6			// xspacc[]>SPECIALACC means special accidental symbol
	INT8	Xunused[6];			// unused bytes in this node
} SPACC_node;
#define SPACC_sizes "111111111111111111"



//								=== NEST ===
//
// If a note is a nested tuplet, the nested outer tuplet specification is stored in the
// NEST node.
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is NEST
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT8	Xnest_tupl;			// 3 for nested triplets  see xnest_intimeof field below
	INT8	Xnest_intimeof;		// used with xnest_tupl field: "xnest_tupl in the time of xnest_intimeof";
								// examples: "3 in the time of 2" or "5 in the time of 7";
								// this field is shifted left by 2 to allow things like
								// "5 quarters in the time of 7 eighths" expressed by
								// "5 in the time of 3.5"
	INT8	Xunused[20];		// unused bytes in this node
} NEST_node;
#define NEST_sizes "11"


//								=== COLOR ===
//
// If a COLOR node precedes a NOTE, the noteheads (or rest) will be displayed
// in the specified color.
//

typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is COLOR
	INT8	Xcontext;			// defines the notation contexts this node applies to

	RGBColor Xxxcolor;			// OUTDATED color: red, green, blue  2 bytes each, 6 total
	INT8	Xnote_color;		// index into color table; standard and alternate coloration
	INT8	Xunused[15];		// unused bytes in this node
} COLOR_node;
#define COLOR_sizes "112221"




//								=== STMOD ===
//
// If a STMOD node precedes a NOTE, the note's slur or tie will be modified by the
// STMOD specifications.  These allow special ties and slurs, such as those required
// for marking a slurred-to note at the beginning of a second ending.  This also
// provides for a second slur, as needed for a trill that is under a long slur and also
// requires a slur to nachschlag notes (after-the-note grace notes).
//

typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is SLURMOD
	INT8	Xcontext;			// defines the notation contexts this node applies to

	UNINT8	XhangRslur : 1;		// break slur drawn to right
	UNINT8	XhangLslur : 1;		// break slur drawn to left
	UNINT8	Xleftonlyslur : 1;	// no slur to right at all (hangRslur ignored for this)

	UNINT8	XhangRtie : 1;		// break tie drawn to right
	UNINT8	XhangLtie : 1;		// break tie drawn to left
	UNINT8	Xleftonlytie : 1;	// no tie to right at all (hangRtie ignored for this)

	UNINT8	Xsplitslur : 1;		// slur split at note (slur ends, another starts at notehead)
	UNINT8	Xaltslursize : 1;	// xslur field refers to slur of type xgrace^xaltslursize


	// Info for additional slur.  The additional slur is only present if xa_slur != NONE.
	// The xa_slur is of the opposite size (grace-type) of the xslur.
	UNINT8	Xa_slur : 2;		// xslur for additional slur
	UNINT8	Xa_crawlslur : 1;	// xcrawlslur for additional slur
	UNINT8	Xa_hangRslur : 1;	// xhangRslur for additional slur
	UNINT8	Xa_hangLslur : 1;	// xhangLslur for additional slur
	UNINT8	Xa_leftonlyslur:1;	// xleftonlyslur for additional slur
	UNINT8	Xa_splitslur : 1;	// xsplitslur for additional slur
	UNINT8	Xa_dashslur : 1;	// xdashslur for additional slur

	UNINT8	Xa_bendslur : 1;	// xbendslur for additional slur

	// Staccato modifier.
	UNINT8	Xstacissimo : 1;	// if there is a staccato dot, it is staccatissimo
	UNINT8	notusedyet : 6;
} TSMOD_node;
#define TSMOD_sizes "11(11111111)(2111111)(116)"






//								=== GENERIC ===
// The GENERIC node is defined as a programming convenience; no nodes of kind
// GENERIC actually occur in Tilia files.  To access a node of unknown kind, the
// code such as the following can be used to test the kind of the node and do a
// C typecast.
//		GENERIC_node *g;
//		NOTE_node *n;
//		...
//		if (generic_node->Xkind == NOTE) {
//			n = (note_node)g;
//			... }
//
typedef struct {
	LINKS						// macro optionally makes next/back pointers for Tilia nodes
	UNINT8	Xkind;				// kind of node; value is anything
	INT8	Xcontext;			// defines the notation contexts this node applies to

	INT8	Xbyte[DATA_BYTES_IN_A_TILIA_NODE];
} GENERIC_node;


#endif
